#!/bin/sh
# anti-autoconf builder - pancake@phreaker.net
# Check for libraries and build sources.

# Set main variables
if [ "${CC}" = "" ];  then CC="gcc";  fi
if [ "${CXX}" = "" ]; then CXX="g++"; fi
FLTK=1
SOUND=1
echo "" > config.log

# Check for Compiler.
printf "[*] Checking for C compiler (${CC})...\t\t\t"
${CC} -v >> config.log 2>&1
if [ "$?" = "0" ]; then echo OK
else	echo "NOT FOUND" ; exit 1; fi

printf "[*] Checking for C++ compiler (${CXX})...\t\t\t"
${CXX} -v >> config.log 2>&1
if [ "$?" = "0" ]; then echo OK
else	echo "NOT FOUND" ; exit 1; fi

if [ ! "${CFLAGS}" = "" ]; then
	echo "[i] Using CFLAGS=\"${CFLAGS}\""
fi
if [ ! "${LDFLAGS}" = "" ]; then
	echo "[i] Using LDFLAGS=\"${LDFLAGS}\""
fi

# Check for FLTK.
printf "[*] Checking for FLTK libs...\t\t\t\t"
cat<<EOF > /tmp/fltk.cxx
#include <FL/Fl.H>
main(){}
EOF
${CXX} ${LDFLAGS} ${CFLAGS} /tmp/fltk.cxx -lfltk -lfltk_images >> config.log 2>&1
if [ "$?" = "0" ]; then
	echo OK
	FLTK=1
else
	echo NOT FOUND
	FLTK=0
fi
rm -f /tmp/fltk.cxx a.out

# Check for OpenGL
printf "[*] Checking for Graphical libs (OpenGL, SDL, GLU)...\t"
cat<<EOF > /tmp/opengl.c
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
main(){}
EOF
${CC} ${LDFLAGS} ${CFLAGS} /tmp/opengl.c -lGL -lSDL -lGLU -lpthread >> config.log 2>&1
if [ "$?" = "0" ]; then echo OK
else	echo "NOT FOUND" ; exit 1; fi
rm -f /tmp/opengl.c a.out

# Check for OpenAL
printf "[*] Checking for OpenAL sound...\t\t\t"
cat<<EOF > /tmp/openal.c
#include <AL/al.h>>
main(){}
EOF
${CC} ${CFLAGS} ${LDFLAGS} /tmp/openal.c -lopenal >> config.log 2>&1
if [ "$?" = "0" ]; then echo "OK"; SOUND=1
else	echo "NOT FOUND" ; SOUND=0 ; fi
rm -f /tmp/openal.c a.out

# Check for Bzip2
printf "[*] Checking for bzlib compression...\t\t\t"
cat<<EOF > /tmp/bzlib.c
#include <bzlib.h>>
main(){}
EOF
${CC} ${CFLAGS} ${LDFLAGS} /tmp/bzlib.c -lbz2 >> config.log 2>&1
if [ "$?" = "0" ]; then echo "OK"; BZLIB=1
else	echo "NOT FOUND" ; BZLIB=0 ; fi
rm -f /tmp/bzlib.c a.out

# Get PREFIX
printf "[i] Using PREFIX from env...\t\t\t\t"
if [ "${PREFIX}" = "" ]; then PREFIX=/usr/local/ ; fi
echo ${PREFIX}
echo ${PREFIX} > PREFIX

#  __  _ _  _  _    __
# | .)| | || || |_ | .\
# |__)|___||_||___||__/
#
echo ""
echo "...sleeping 2 seconds..."
echo ""
sleep 2
echo "[*] All checks done. Go build!"
echo ""
if [ ${FLTK} = 1 ]; then
	printf "\e[33m[*] Building installer...\e[0m\n"
	cd glhi/src/
	make all
	cd -
	printf "\e[33m[*] Building launcher...\e[0m\n"
	cd mainh/src/
	make all
fi
printf "\e[33m[*] Building networking...\e[0m\n"
	cd networkh
	make all
	cd -
printf "\e[33m[*] Building 3D engine...\e[0m\n"
	if [ ${SOUND} = 1 ]; then 
		export CFLAGS="${CFLAGS} -DOPENAL=${SOUND} -DPREFIX=\\\"${PREFIX}\\\"" 
		export LDFLAGS="${LDFLAGS} -lopenal" ; fi
	if [ ${BZLIB} = 1 ]; then
		export CFLAGS="${CFLAGS} -DBZLIB=1 -DPREFIX=\\\"${PREFIX}\\\""
		export LDFLAGS="${LDFLAGS} -lbz2" ; fi
	cd engineh/src
	make all
	cd -
printf "\e[33m[*] Building developer-tools...\e[0m\n\n"
	cd dev-tools
	sh build.sh all
	cd -

printf "\e[33m[*] ALL DONE.\e[0m\n"

exit 0;
