#include "Camera.h"

Camera::Camera(char *ruta)
{
	HUD = new Interface();
	DNI = -1;
	yawH = 0.0f;
	yawV = 0.0f;
	load(ruta);
	idLegs = this->getControl(LEGS)->getObject3D()->getTexture();
	idBody = this->getControl(BODY)->getObject3D()->getTexture();
	idHead = this->getControl(HEAD)->getObject3D()->getTexture();
	isThirdView = false;	
}

Camera::~Camera()
{
	delete HUD;
}

GL3f Camera::getObjPos()
{
	GL3f objCam;

	if (yawV>1)  yawV =  1;
	if (yawV<-1) yawV = -1;

	// set camera position
	objCam.x = getPos().x+(float)cos(yawH)*(float)cos(yawV);
	objCam.y = getPos().y+(float)sin(yawV);
	objCam.z = getPos().z+(float)sin(yawH)*(float)cos(yawV);

	return objCam;
}

void Camera::setObj(double yawH, double yawV)
{
	this->yawH = yawH;
	this->yawV = yawV;
}

void Camera::setObj(GL3f vector)
{
	/* FYI : http://itc.utk.edu/itc/grants/twt2000/modules/mbreinig/3dcoordinates.htm */
	yawH = atan( vector.y / vector.x );
	yawV = atan( (double)vector.z / sqrt((double)(vector.x*vector.x)+(double)(vector.y*vector.y)) );
}

float Camera::getRotation()
{
	return yawH;
}

GL2f Camera::getYaw()
{
	return GL2f(cos(yawH)*cos(yawV),sin(yawH)*sin(yawV));
}

int Camera::getDNI()
{
	return DNI;
}

void Camera::setDNI(int DNI)
{
	this->DNI=DNI;
}

Interface* Camera::getInterface()
{
	return HUD;
}

void Camera::setThirdView (bool state)
{
	isThirdView=state;
}

bool Camera::getThirdView()
{
	return isThirdView;
}

void Camera::swapThirdView()
{
	isThirdView^=1;
}

void Camera::centerV()
{
	yawV = 0.0f;
}

void Camera::setOrientation(double yawH, double yawV)
{
	setObj(yawH,yawV);
}

void Camera::addOrientation (Orientation *orientation)
{
	if (this->yawH >= 2*M_PI) 
		this->yawH -= 2*M_PI; else
	if (this->yawH <= 0.0f)
		this->yawH += 2*M_PI;
 
	this->yawH += orientation->getH();
	this->yawV += orientation->getV();

	if (isThirdView)
	{
		this->yawV = 0.0f;
	}
	else
	{
		if (this->yawV >=  (M_PI/2)*MAX_FACTOR)
		{
			this->yawV =  (M_PI/2)*MAX_FACTOR;
		} else
		if (this->yawV <= -(M_PI/2)*MAX_FACTOR) 
		{
			this->yawV = -(M_PI/2)*MAX_FACTOR;
		}
	}

}

void Camera::addMovement(Movement *mov)
{
	setPos(getPos()+mov->pos);
}
