/*! \file  Controls.cpp
 *  \brief Controls class definition
 */

#include "Controls.h"

/*!
 * Loads all the device controllers.
 *
 * \param stage The Stage we are currently playing
 * \param net   Our Networkh connection
 */
Controls::Controls (Camera *cam, Stage *stg, Collision *col, Networkh *net) 
{
	ctrlMs = new ControlsMouse(cam);
	ctrlKb = new ControlsKeyboard(cam, stg, col, net);
	finished = false;
}

Controls::~Controls()
{
	delete ctrlMs;
	delete ctrlKb;
}

/*!
 * Makes all the device controllers work.
 */
void Controls::input()
{
	SDL_PumpEvents();

	// first take a look on the mouse input, as moving the character can
	// infuence on external (third person) camera position (to avoid
	// trespassing walls)
	ctrlMs->input();
	ctrlKb->input(&finished);
}

