/*! \file  ControlsKeyboard_defaults.cpp
 *  \brief Defaul keyboard key associations
 *
 *  This file contains the "key translation table" from the glH functional keys
 *  to its SDL-equivalent keys, the previous key state (pressed/released) and
 *  its associated callback.
 *
 *  The order in the declaration will induce the order of aplication of the
 *  different callbacks (the first declared, the first applicated)
 */

const ControlsKeyboard::_key_t ControlsKeyboard::default_keys[] = {
	{"WALK_F",	SDLK_w,		KEY_RELEASED, &ControlsKeyboard::f_walk_f 	},
	{"WALK_B",	SDLK_s,		KEY_RELEASED, &ControlsKeyboard::f_walk_b 	},
	{"RUN",		SDLK_LSHIFT,	KEY_RELEASED, &ControlsKeyboard::f_run 		},
	{"TURN_L",	SDLK_a,		KEY_RELEASED, &ControlsKeyboard::f_turn_l 	},
	{"TURN_R",	SDLK_d,		KEY_RELEASED, &ControlsKeyboard::f_turn_r 	},
	{"STRAFE_L",	SDLK_q,		KEY_RELEASED, &ControlsKeyboard::f_strafe_l 	},
	{"STRAFE_R",	SDLK_e,		KEY_RELEASED, &ControlsKeyboard::f_strafe_r 	},
	{"JUMP",	SDLK_SPACE,	KEY_RELEASED, &ControlsKeyboard::f_jump 	},
	{"CENTER_V",	SDLK_c,		KEY_RELEASED, &ControlsKeyboard::f_center_v 	},
	{"LOOK_UP",	SDLK_3,		KEY_RELEASED, &ControlsKeyboard::f_look_up 	},
	{"LOOK_DOWN",	SDLK_4,		KEY_RELEASED, &ControlsKeyboard::f_look_down 	},
	{"FIRE",	SDLK_f,		KEY_RELEASED, &ControlsKeyboard::f_fire 	},
	{"CHWEAP",	SDLK_v,		KEY_RELEASED, &ControlsKeyboard::f_chweap 	},
	{"MAP",		SDLK_m,		KEY_RELEASED, &ControlsKeyboard::f_map 		},
	{"TALK",	SDLK_t,		KEY_RELEASED, &ControlsKeyboard::f_talk 	},
	{"SCREENSHOT",	SDLK_F9,	KEY_RELEASED, &ControlsKeyboard::f_screenshot 	},
	{"PAUSE",	SDLK_o,		KEY_RELEASED, &ControlsKeyboard::f_pause 	},
	{"QUIT",	SDLK_ESCAPE,	KEY_RELEASED, &ControlsKeyboard::f_quit 	},
	{"FOVp",	SDLK_PLUS,	KEY_RELEASED, &ControlsKeyboard::f_fovp 	},
	{"FOVm",	SDLK_MINUS,	KEY_RELEASED, &ControlsKeyboard::f_fovm 	},
	{"SHOWPOS",	SDLK_x,		KEY_RELEASED, &ControlsKeyboard::f_showpos 	},
	{"SHOWITEMS",	SDLK_i,		KEY_RELEASED, &ControlsKeyboard::f_showitems 	},
	{"SHOWTHIRDP",	SDLK_p,		KEY_RELEASED, &ControlsKeyboard::f_showthirdp 	},
	{"KILL",	SDLK_k,		KEY_RELEASED, &ControlsKeyboard::f_kill 	},
	{"MODE_PORTAL",	SDLK_F1,	KEY_RELEASED, &ControlsKeyboard::f_mode_portal 	},
	{"MODE_BOUNDING",SDLK_F2,	KEY_RELEASED, &ControlsKeyboard::f_mode_bounding},
	{"MODE_PAINTING",SDLK_F3,	KEY_RELEASED, &ControlsKeyboard::f_mode_painting},
	{"MODE_TEXTURES",SDLK_F4,	KEY_RELEASED, &ControlsKeyboard::f_mode_textures},
	{"MODE_FOG",	SDLK_F5,	KEY_RELEASED, &ControlsKeyboard::f_mode_fog 	},
	{"MODE_LIGHT",  SDLK_F6,        KEY_RELEASED, &ControlsKeyboard::f_mode_light   },
	{"MODE_COL",    SDLK_F7,        KEY_RELEASED, &ControlsKeyboard::f_mode_col     },
	{"FLY_UP",	SDLK_1,		KEY_RELEASED, &ControlsKeyboard::f_fly_up 	},
	{"FLY_DOWN",	SDLK_2,		KEY_RELEASED, &ControlsKeyboard::f_fly_down 	}
};

