#include "OpenGLH.h"
#include "Fog.h"

Fog::Fog()
{
	Fog(0,0,0,0);
}

Fog::Fog(int mode,float start,float end,float dens,
	 int red, int green, int blue, int alpha)
{
// TODO use floats for performance
	if (end<0)     end   = -end;
	if (dens<0)    dens  = -dens;
	if (red>255)   red   = 255;
	if (red<0)     red   = 0;
	if (green>255) green = 255;
	if (green<0)   green = 0;
	if (blue>255)  blue  = 255;
	if (blue<0)    blue  = 0;
	if (alpha>255) alpha = 255;
	if (alpha<0)   alpha = 0;

	m = FogMode    = mode;
	e = FogEnd     = end;
	d = FogDensity = dens;
	r = FogRed     = red;
	g = FogGreen   = green;
	b = FogBlue    = blue;
	a = FogAlpha   = alpha;

	noth = 0;
	oth  = NULL;
	pos  = NULL;
}

Fog::~Fog()
{
	for (int i=0;i<noth;i++)
		delete oth[i];
	free(oth);
	free(pos);
}



void Fog::add(Fog *f, GL3f xyz)
{
	noth++;
	oth=(Fog**)realloc(oth,sizeof(Fog*)*noth);
	oth[noth-1]=f;
	pos=(GL3f*)realloc(pos,sizeof(GL3f)*noth);
	pos[noth-1]=xyz;
}

void Fog::begin(GL3f mypos)
{
	GL3f dis;
	float mod;

	if (!FOG)
	{
		doBegin();
		return;
	}

	if (FogMode!=0)
	{
		m=FogMode;
		e=FogEnd;
		d=FogDensity;
		doBegin();
		return;
	}

	/* init */
	m=0;
	s=e=d=0;
	r=g=b=a=0;
	float mult;

	for (int i=0;i<noth;i++)
	{
		dis = pos[i]-mypos;
		mod = sqrtf( (dis.x*dis.x)+(dis.y*dis.y)+(dis.z*dis.z) );

		mult = 1-(mod / (oth[i]->e) );

		e = (e+mod)/2;
		d += (mult) * oth[i]->d;
		/* colors */
		r += (int)(mult * oth[i]->r);
		g += (int)(mult * oth[i]->g);
		b += (int)(mult * oth[i]->b);
			
		//ds = mod-oth[i]->s;
		//de = oth[i]->e-mod;

		s=0;
		e=(1-mult) *mod;

		// set mode
		if (oth[i]->m > m) m=oth[i]->m;
	}

	doBegin();
}

void Fog::doBegin()
{

	if (!FOG || m==0)
	{
		glDisable(GL_FOG);
		return;
	}

	glEnable(GL_FOG);

	switch(m)
	{
	case 1:
        	glFogi(GL_FOG_MODE,GL_LINEAR);
		break;
	case 2:
		glFogi(GL_FOG_MODE,GL_EXP);
		break;
	}

	GLfloat color[4] = {
		(float)r/255,
		(float)g/255,
		(float)b/255,
		(float)a/255 };

	glFogfv(GL_FOG_COLOR,color);
        glFogf(GL_FOG_START,s);
        glFogf(GL_FOG_END,e);
        glFogf(GL_FOG_DENSITY,d);
}

void Fog::end()
{
	if (FogMode>0)
	glDisable(GL_FOG);
}
