#include "Light.h"

int LIGHT=1;

Light::Light()
{
	for(int i=0;i<8;i++)
	{
		glDisable(GL_LIGHT0+i);
		lights[i]=false;
	}
	smooth();
        GLfloat amb[]={1.0,1.0,1.0,1};
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, amb);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_LIGHTING);
}

Light::~Light()
{
	disable();
}

int Light::add()
{
	for (int i=0;i<8;i++)
		if (lights[i]==false)
		{
			glEnable(GL_LIGHT0+i);
			lights[i]=true;
			/* set default values */
			float difu[]={0.8,0.8,0.8,1};
			glLightfv(GL_LIGHT0+i,GL_DIFFUSE,difu);
		        float ambi[]={0.6,0.6,0.6,1};
       			glLightfv(GL_LIGHT0+i,GL_AMBIENT,ambi);
        		float espe[]={0.7,0.7,0.7,1};
        		glLightfv(GL_LIGHT0+i,GL_SPECULAR,espe);
			return i;
		}
	return -1;
}

void Light::
del(int lid)
{
	if (checkLid(lid)) return;
	lights[lid]=false;
	glDisable(GL_LIGHT0+lid);
}

void Light::
setPosition(int lid, XYZ)
{
	if (checkLid(lid)) return;
	
	pos[lid*3+0]=x;
	pos[lid*3+1]=y;
	pos[lid*3+2]=z;


	glLightfv(GL_LIGHT0+lid, GL_POSITION, pos+lid*3);
}

void Light::
setPosition(int lid,GL3f xyz)
{
	setPosition(lid,xyz.x,xyz.y,xyz.z);
}

void Light::
update(GL3f xyz,float yaw)
{
	float np[3];

	for (int i=0;i<8;i++)
	{
		if (lights[i])
		{
		memcpy(np,pos+i*3,3*sizeof(float));
//		printf("-- %f %f %f\n",xyz.x,xyz.y,xyz.z);
		np[0]=xyz.x-np[0];
		np[1]-=xyz.y;
		np[2]-=xyz.z;
		np[0]*=sinf(yaw);
		np[2]*=cosf(yaw);
//		printf("++ %f %f %f\n",np[0],np[1],np[2]);
		glLightfv(GL_LIGHT0+i,GL_POSITION,np);
		}
	}
}

void Light::
setDirection(int lid, XYZ)
{
	if (checkLid(lid)) return;
	float dir[]={x,y,z};
	glLightfv(GL_LIGHT0+lid, GL_SPOT_DIRECTION, dir);
}

void Light::
setCutOff(int lid, float value)
{
	if (checkLid(lid)) return;
	glLightf(GL_LIGHT0+lid,GL_SPOT_CUTOFF,value);
}

void Light::
enable(int lid)
{
	if (checkLid(lid)) return;

	glEnable(GL_LIGHT0+lid);
}

void Light::
enable()
{
	glEnable(GL_LIGHTING);
	for (int i=0;i<8;i++)
		if (lights[i]==true)
			glEnable(GL_LIGHT0+i);

}

void Light::
disable(int lid)
{
	if (checkLid(lid)) return;

	glDisable(GL_LIGHT0+lid);
}

void Light::
disable()
{
	for (int i=0;i<8;i++)
		glDisable(GL_LIGHT0+i);

	glDisable(GL_LIGHTING);
}

void Light::
setDiffuse(int lid, RGBA)
{
	if (checkLid(lid)) return;

	float rgba[]={r,g,b,a};
	glLightfv(GL_LIGHT0+lid,GL_DIFFUSE,rgba);
}

void Light::
setSpecular(int lid, RGBA)
{
	if (checkLid(lid)) return;

	float rgba[]={r,g,b,a};
	glLightfv(GL_LIGHT0+lid,GL_SPECULAR,rgba);
}

void Light::
setAmbient(int lid, RGBA)
{
	if (checkLid(lid)) return;

	float rgba[]={r,g,b,a};
	glLightfv(GL_LIGHT0+lid,GL_AMBIENT,rgba);
}

void Light::
setAmbient(RGBA)
{
	float rgba[]={r,g,b,a};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, rgba);
}

bool Light::
checkLid(int lid)
{
	if (lid<0) return true;
	if (lid>7) return true;
	return false;
}

void Light::
flat()
{
	glShadeModel(GL_FLAT);
}

void Light::
smooth()
{
	glShadeModel(GL_SMOOTH);
}
