#include "MainMenu.h"

MainMenu::MainMenu()
{
	m = new Menu();
	m->addButton(0.12,0.57,0.32,0.62);
	m->addButton(0.12,0.64,0.32,0.68);
	m->addButton(0.12,0.70,0.32,0.74);
	m->addButton(0.12,0.76,0.32,0.80);
	m->addButton(0.12,0.82,0.32,0.86);

	keypressed=false;

	cursorX=cursorY=0;
	option=0;
	esc=false;

	/* init opengl */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90.0,0.0,0.0,500.0);

	/* modelview */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LESS);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glDisable(GL_FOG);

	/* camera */
	gluLookAt(
		0,0,-100,
		0,0,0,
		0,1,0);

	/* load textures */
	background = TGAloader_tid("mainmenu.jpg");
        mouse = TGAloader_tid("mouse.png");
}

MainMenu::~MainMenu()
{
	/* free textures */
	TGAloader_untid(mouse);
	TGAloader_untid(background);
	delete m;
}

void MainMenu::drawScene()
{
        cursorX=2*(cursorX/WIDTH)-1;
        cursorY=1-(2*(cursorY/HEIGHT));

	/* scene */
	glClearColor(0.0,0.0,0.0,1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	drawImage(background);

	/* init opengl */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90.0,0.0,0.0,500.0);

	/* modelview */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	/* load textures */
	glLoadIdentity();

	/* selector */
	glDisable(GL_TEXTURE_2D);
	glColor3f(1,1,1);
	glBegin(GL_TRIANGLES);
		glColor3f(1.0,1.0,1.0);
		glTexCoord2f(1,0.5);
		glVertex3f(-0.8,  -0.2-option*0.12,  0.0); /* mr */
		glTexCoord2f(0,0);
		glVertex3f(-0.82, -0.18-option*0.12,  0.0); /* tl */
		glTexCoord2f(0,1);
		glVertex3f(-0.82, -0.22-option*0.12,  0.0); /* dl */
	glEnd();

	glEnable(GL_TEXTURE_2D);
	/* cursor */
	glPushMatrix();
	glTranslatef(cursorX,cursorY,-0.1);
	glBindTexture(GL_TEXTURE_2D,mouse);
	glBegin(GL_TRIANGLES);
	glTexCoord2f(0,0);
	glVertex3f(0.0,0.0,-0.1);
	glTexCoord2f(0,1);
	glVertex3f(0.0,-0.05,-0.1);
	glTexCoord2f(1,1);
	glVertex3f(0.05,-0.05,-0.1);
	glEnd();
	glPopMatrix();
}

void MainMenu::setOption(int n)
{
	option=n;
	if (option<0) option=MENU_LAST-1;
	if (option==MENU_LAST) option=0;
}

void MainMenu::getKeys()
{
	Uint8 *keys;
	SDL_Event ev;
	int x,y;

	/* fetch user events */
	SDL_PumpEvents();
	SDL_GetMouseState(&x,&y);
	keys=SDL_GetKeyState(NULL);
	SDL_PollEvent(&ev);
	cursorX=x;
	cursorY=y;

	int over = m->isOverItem(x,y);
	if (over != -1)
	{
		setOption(over);
		if (ev.type == SDL_MOUSEBUTTONUP)
			esc=true;
	}

	if (keys[SDLK_1])
		{
		esc=true;
		setOption(MENU_OFFLINE);
		}
	if (keys[SDLK_ESCAPE])
		{
		esc=true;
		setOption(MENU_EXIT);
		}

	if (keys[SDLK_RETURN])
		{
		esc=true;
		}

	if (keypressed==false)
	{
	if (keys[SDLK_UP]==SDL_PRESSED)
		{
		setOption(option-1);
		keypressed=true;
		}
	if (keys[SDLK_DOWN]==SDL_PRESSED)
		{
		setOption(option+1);
		keypressed=true;
		}
	}
	if (! keys[SDLK_UP] && !keys[SDLK_DOWN] )
	{
		keypressed=false;
	}
}

bool MainMenu::isFinished()
{
	return esc;
}

int MainMenu::getOption()
{
	printf("OPTION: %d\n",option);
	return option;
}

