#include "Map.h"

/* CLASS */
Map::Map()
{
	nRooms = 0;
	numTextures   = 0;
	textureArray  = 0;
	materialArray = 0;
	rooms = 0;
	skyFog = 0;
	walls  = NULL;
	nWalls = 0;
	nps_added = nps = 0;
	ps = NULL;
	fondo = new Sky();

	try
	{
		sound = NULL;
		soundRef = NULL;
		sound = new Audioh(DEBUG);
		sound->setVolume(SOUND_VOLUME);

	} catch(int err) {
		printf("[E] Cannot initialize audio (%d)\n",err);
	}
}

Map::~Map()
{
	for (int i=0;i<numTextures;i++)
		TGAloader_untid(textureArray[i].idTex);
	delete[] textureArray;
	for (int i=0;i<nRooms;i++)
		delete rooms[i];

	delete[] rooms;
	delete fondo;
	delete skyFog;
	delete sound;
	free(soundRef);

#if 0
	if (ps)
	for(int i=0;i<nps_added;i++)
		delete ps[i];
	delete[] ps;

	free (walls);
#endif
	printf("[<] Map\n");
}

/* COLLISION */
void Map::addCollision(
	float x1, float y1, float z1,
	float x2, float y2, float z2,
	float x3, float y3, float z3 )
{
	int i = nWalls*9;
	nWalls++;
	walls=(float *)realloc(walls, (i+9)*sizeof(float));

	walls[i+0]=x1;
	walls[i+1]=y1;
	walls[i+2]=z1;

	walls[i+3]=x2;
	walls[i+4]=y2;
	walls[i+5]=z2;

	walls[i+6]=x3;
	walls[i+7]=y3;
	walls[i+8]=z3;
}

void Map::createParticleSystems(int n)
{
	delete[] ps;
	ps = new ParticleSystem*[n];
	nps = n;
}

bool Map::addParticleSystem(char *file, float x, float y, float z)
{
	if (nps_added>=nps)
		return false;
	ps[nps_added] = new ParticleSystem(file);
	ps[nps_added]->setPosition(x,y,z);
	ps[nps_added]->start();
	nps_added++;
	return true;
}

void Map::drawParticleSystems(float SIN,float COS)
{
	glPushMatrix();
	for (int i=0;i<nps_added;i++)
	{
		ps[i]->update();
		ps[i]->draw(SIN,COS);
	}
	glPopMatrix();
}

bool Map::endParticleSystems()
{
	return (nps == nps_added);
}

GLFace *Map::getCollision(int n)
{
	if (n<0||n>=nWalls)
		return NULL;
/*
printf("# %d: %f %f %f - %f %f %f - %f %f %f\n",n,
	walls[0+n*9],walls[1+n*9],walls[2+n*9],
	walls[3+n*9],walls[4+n*9],walls[5+n*9],
	walls[6+n*9],walls[7+n*9],walls[8+n*9]);
*/
	return new GLFace(
		GL3f(walls[0+n*9],walls[1+n*9], walls[2+n*9]),
		GL3f(walls[3+n*9],walls[4+n*9], walls[5+n*9]),
		GL3f(walls[6+n*9],walls[7+n*9], walls[8+n*9]) );
}
	
void Map::drawCollision()
{
	int i,j;

	if (FOG)
	glDisable(GL_FOG);
	glDisable(GL_CULL_FACE); // draw both faces

	glPolygonMode(GL_FRONT, GL_FILL);

	for (i=0;i<nWalls;i++)
	{
		j=i*9;
		glColor3f(1,0,0);
		glBegin(GL_TRIANGLES);
		glVertex3f(walls[j+0],walls[j+1],walls[j+2]);
		glVertex3f(walls[j+3],walls[j+4],walls[j+5]);
		glVertex3f(walls[j+6],walls[j+7],walls[j+8]);
		glEnd();
	}
	if (FOG)
	glEnable(GL_FOG);
	if (CULL)
	glEnable(GL_CULL_FACE); // draw once
}

/* FOG */
void Map::setSkyFog(
	int mode, 
	float start, float end, float density,
	int r, int g, int b, int a)
{
	delete skyFog;
	skyFog = new Fog(mode,end,density,r,g,b,a);
}

void Map::addFog(
	int mode,
	float range, float density,
	int r, int g, int b, int a,
	float x, float y, float z )
{
	if (skyFog)
	{
		Fog *f = new Fog(mode,0,range,density,r,g,b,a);
		skyFog->add(f,GL3f(x,y,z));
	}
}

void Map::endFog()
{
	if (skyFog)
	skyFog->end();
}

void Map::beginFog(GL3f xyz)
{
	if (skyFog)
	skyFog->begin(xyz);
}

/* ROOMS */
int Map::getNumRooms()
{
	return nRooms;
}

Room *Map::getRoom(int pos)
{
	if (pos>=0 && pos<nRooms)
		return rooms[pos];

	return NULL;
}

bool Map::setTexture(int num, char *textura)
{
	unsigned int tid;
	if (num<0 || num>=numTextures) return false;

	textureArray[num] = texture();

	tid = TGAloader_tid ( textura, &textureArray[num].bpp );

	if (tid==0)
	{
		printf("Map: ERROR LOADING TEXTURE %s\n",textura);
		return false;
	}

	textureArray[num].idTex = tid;

	/* all ok */
	return true;
}

texture Map::getTexture(int pos)
{
	if (pos>=0 && pos<numTextures)
		return textureArray[pos];
	return 	texture();
}

bool Map::createRooms(int num)
{
	if (num<=0) return 0;
	rooms=new Room*[num];
	if (!rooms) return false;
	nRooms=num;
	printf(" |- Rooms: %d\n",nRooms);

	return true;
}

bool Map::createTextures(int num)
{
	if (num<=0) return false;

	numTextures=num;

	textureArray = new texture[num];
	materialArray = new Material*[num];

	for (int i=0;i<num;i++)
		materialArray[i] = new Material();

	return true;
}

int Map::addRoom(int id)
{
	if (id<0 || id>=nRooms) return 0;
	rooms[id] = new Room(id);
	if (!rooms[id]) return 0;

	return 1;
}

int Map::getNumTextures()
{
	return numTextures;
}

Sky *Map::getSky()
{
	return fondo;
}

int Map::loadSky(char *ruta)
{
	return fondo->loadSky(ruta);
}

int Map::loadSound(char *file)
{
	FILE* fd;
	char buf[1024];
	int hab;
	int i,j,k;
	bool newSound=true;

	if (sound && SOUND_ENABLED)
	{
	walkSound = new int[nRooms];
	runSound = new int[nRooms];

	chdir(getDirs(file));
	fd=fopen (file,"r");

	if (!fd) return 0;

	fscanf(fd,"%s",buf);
	if (strcmp("AMBIENT_SOUND",buf))
	{
		printf("Missing AMBIENT_SOUND in %s\n",file);
		fclose(fd);
		return 0;
	}
	fscanf(fd,"%s",buf);

	soundRef=(int *)realloc(soundRef,(sound->getNumBuffers()+1)*sizeof(int));

	chdir(getDirs(buf));
	soundRef[0]=sound->add(buf); // XXX SOUND

	ambientSound=sound->sid_new(soundRef[0]);
	sound->setListener(0,0,0);
	sound->sid_pos(ambientSound,0,0,0);
	sound->sid_play(ambientSound);
	sound->sid_loop(ambientSound,true);

	for ( i=0;i<nRooms && !feof(fd);i++)
	{
		fscanf(fd,"%s %d",buf,&hab);

		for (j=0;j<2 && !feof(fd);j++)
		{
			fscanf(fd,"%s",buf);
			fscanf(fd,"%s",buf);
			for ( k=0; k<sound->getNumBuffers(); k++)
			{
				char *name = NULL;
				name = sound->getFileName(k);
				if (name != NULL)
				{
					if ( !strcmp(buf,name) )
					{
						newSound = false;
						break;
					}
				}
			}
			if (newSound)
			{
				soundRef=(int *)realloc(soundRef,
				(sound->getNumBuffers()+1)*sizeof(int *));

				chdir(getDirs(buf));
				soundRef[k]=sound->add(buf);

				if (!j) walkSound[i]=sound->sid_new(soundRef[k-1]);
				else runSound[i]=sound->sid_new(soundRef[k-1]);
			} else {
				if (!j) walkSound[i]=sound->sid_new(soundRef[k]);
				else runSound[i]=sound->sid_new(soundRef[k]);
				newSound=true;
			}
		}
	}
	fclose(fd);

	} // SOUND_ENABLED

	return 1;
}

int Map::playSound(int hab, int mode, GL3f pos)
{
	if (sound && SOUND_ENABLED)
	{
		if (hab<0 || hab >nRooms) return 0;

		sound->setListener(pos.x,pos.y,pos.z);
		sound->sid_pos(ambientSound,pos.x,pos.y,pos.z);

		switch(mode)
		{
		case RUNNING_FORWARD:
			sound->sid_pos(runSound[hab],pos.x,pos.y,pos.z);
			if (sound->sid_state(runSound[hab])!=SOUND_PLAY)
			{
				for (int i=1;i<sound->getNumSources();i++)
					if (sound->sid_state(i)==SOUND_PLAY)
					sound->sid_stop(i);

				sound->sid_play(runSound[hab]);
			}
			break;
		case WALKING_FORWARD:
			sound->sid_pos(walkSound[hab],pos.x,pos.y,pos.z);
			if (sound->sid_state(walkSound[hab])!=SOUND_PLAY)
			{
				for (int i=1;i<sound->getNumSources();i++)
					if (sound->sid_state(i)==SOUND_PLAY)
					sound->sid_stop(i);

				sound->sid_play(walkSound[hab]);
			}
			break;
		default:
			return 0;
		}
	}
	return 1;
}

void Map::stopSound()
{
	if (sound && SOUND_ENABLED)
	{
	for (int i=1;i<sound->getNumSources();i++)
		if (sound->sid_state(i)==SOUND_PLAY)
		sound->sid_stop(i);
	}
}

void Map::setMaterial(int id, int GL, RGBA)
{
	float rgba[4]={r,g,b,a};

	if (id <0 || id > numTextures) return;
	materialArray[id]->setProperty(GL,rgba);
}

void Map::setMaterial(int id, int GL, float shi)
{
	materialArray[id]->setProperty(GL,shi);
}

void Map::useMaterial(int id)
{
	materialArray[id]->draw();
}
