#ifndef _MAP_H_
#define _MAP_H_

#include "Room.h"
#include "TGAloader.h"
#include "Sky.h"
#include "Fog.h"
#include "OpenGLH.h"
#include "Material.h"
#include "Audioh.h"
#include "ParticleSystem.h"
#include "external.h"

//Clase q almacena todo lo relacionado con objetos no mobiles (es escenario,
//los portales, etc.)
class Map
{
	int numTextures;
	texture *textureArray;
	Material **materialArray;
	ParticleSystem **ps;
	int nps, nps_added;

	int nRooms;
	Room **rooms;

	/* FOG */
	Fog *skyFog;

	Sky *fondo;

	Audioh *sound;

	int ambientSound;
	int *soundRef;
	int *walkSound, *runSound;

	int nWalls;
	float *walls;

public:
	/* CDTORS */
	Map();
	~Map();
	
	/* ROOMS */
	int getNumRooms();
	Room *getRoom(int pos);
	//Inicializa el array de habitaciones
	//Espera saber cuantas habitaciones hay
	//Devuelve 1 si todo ha ido bien y 0 en caso contrario
	bool createRooms(int num);

	/* TEXTURES */
	texture getTexture(int pos);
	int getNumTextures();
	bool setTexture(int,char *);
	
	//Inicializa el array de texturas
	//Espera saber cuantas texturas hay
	//Devuelve 1 si todo ha ido bien y 0 en caso contrario
	bool createTextures(int num);

	void createParticleSystems(int n);
	bool addParticleSystem(char *file, float x,float y, float z);
	bool endParticleSystems();
	void drawParticleSystems(float SIN,float COS);
	
	//Inicializa una habitacion
	//Le has de pasar el id q tiene
	//Devuelve 1 si todo ha ido bien y 0 en caso contrario
	int addRoom(int id);

	/* SKY AND FOG */
	Sky* getSky();
	int  loadSky(char *ruta);
	void setSkyFog(int,float,float,float,int r=0, int g=0, int b=0,int a=0);
	void addFog(int,float,float,int,int,int,int, float,float,float);
	void endFog();
	void beginFog(GL3f);

	/* SOUND */
	int loadSound(char *ruta);
	int playSound(int hab, int mode, GL3f pos);
	void stopSound();

	/* COLLISION */
	void addCollision(float,float,float, float,float,float, float,float,float);
	GLFace *getCollision(int n);
	void drawCollision();

	void setMaterial(int,int,RGBA);
	void setMaterial(int,int,float);
	void useMaterial(int);
};

#endif
