#ifndef _OPENGL_H_
#define _OPENGL_H_

#define RGBA float r, float g, float b, float a
#define XYZ  float x, float y, float z

#ifdef WIN32
#include <windows.h>
#endif

#ifdef __APPLE__
#include <gl.h>
#include <glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "Dirs.h"
#include "SDL/SDL.h"
#include "string.h"
#include "external.h"
#include "TGAloader.h"

#include "../../config.h"

//Loads the TGA info
typedef struct 
{
	GLubyte *imageData;
	GLuint bpp;
	GLuint width;
	GLuint height;
	GLuint texID;
} TGA;


//Loads info of the textures
struct texture {
public:
	int tam;				//The texture size
	int width;				//The texture width
	int height;				//The texture size
	int bpp;				//The texture bpp
	unsigned int idTex;		//The texture id

	//Init vars
	texture()
	{
	        height = 0;
		tam   = 0;             bpp    = 0;
		width = 0;             idTex  = 0; 
	}
};

//Loads 2 floats
struct GL2f
{
	float x, y;

	GL2f () { x=y=0.0f; }

	GL2f (float x,float y)
		{ this->x = x;
		  this->y = y; }

	GL2f operator+(GL2f temp)
		{ return GL2f(temp.x + x, temp.y + y); }

	GL2f operator-(GL2f temp)
		{ return GL2f(temp.x - x, temp.y - y); }

	GL2f operator*(float num)
		{ return GL2f(x * num, y * num); }

	GL2f operator/(float num)
		{ return GL2f(x / num, y / num); }
};


//Loads 3 floats
struct GL3f
{
	float x, y, z;

	GL3f()	{ x=y=z=0.0f; }
	
	GL3f(float x,float y, float z)
		{ this->x = x;
		  this->y = y;
		  this->z = z; }

	GL3f(const GL3f& old) {
		x = old.x;
		y = old.y;
		z = old.z;
	}

	GL3f& operator=(const GL3f temp ) {
		x = temp.x;
		y = temp.y;
		z = temp.z;

		return *this;
	}

	bool operator>(float ff){
		return ( (fabsf(x)+fabsf(y)+fabsf(z)) > ff );
	}

	bool operator==(float ff){
		return ( (fabsf(x)+fabsf(y)+fabsf(z)) == ff );
	}

	GL3f operator+(GL3f temp){
		return GL3f(temp.x + x, temp.y + y, temp.z + z);
	}

	GL3f operator+(float num){
		return GL3f(x + num, y + num, z + num);
	}
	
	GL3f operator-(GL3f temp){
		return GL3f(x - temp.x, y - temp.y, z - temp.z);
	}

	GL3f operator*=(GL3f v) {
		return GL3f(x*v.x,y*v.y,z*v.z);
	}

	GL3f operator*(float num){
		return GL3f(x * num, y * num, z * num);
	}

	float operator*(GL3f v2){
		return (x * v2.x +  y * v2.y + z * v2.z);
	}

	GL3f operator/(float num){
		return GL3f(x / num, y / num, z / num);
	}

	float size()
	{
		return sqrtf(x*x+y*y+z*z);
	}

	GL3f normalize()
	{
		return (GL3f)(*this/this->size());
	}

	void print(){
		printf("%f %f %f\n",x,y,z);
	}
};

/* face */
struct GLFace
{
	GL3f a;
	GL3f b;
	GL3f c;
	GLFace(GL3f a,GL3f b, GL3f c)
	{ this->a=a; this->b=b; this->c=c; }

	GL3f normal()
	{
		float x = (b.y-a.y)*(c.z-a.z) - (b.z-a.z)*(c.y-a.y);
		float y = (b.z-a.z)*(c.x-a.x) - (b.x-a.x)*(c.z-a.z);
		float z = (b.x-a.x)*(c.y-a.y) - (b.y-a.y)*(c.x-a.x);
		return GL3f(x,y,z).normalize();
	}
};

//Loads 3 integers
struct GL3i{
	int x;
	int y;
	int z;

	GL3i(){
		x=0;
		y=0;
		z=0;
	}

	GL3i(int x_old,int y_old, int z_old){
		x=x_old;
		y=y_old;
		z=z_old;
	}

	GL3i operator+(GL3i temp){
		return GL3i(temp.x + x, temp.y + y, temp.z + z);
	}
	
	GL3i operator-(GL3i temp){
		return GL3i(temp.x - x, temp.y - y, temp.z - z);
	}

	GL3i operator*(int num){
		return GL3i(x * num, y * num, z * num);
	}

	GL3i operator/(int num){
		return GL3i(x / num, y / num, z / num);
	}
};

//Loads 4 floats
struct GL4f{
	float x;
	float y;
	float z;
	float a;

	GL4f(){
		x=0.0f;
		y=0.0f;
		z=0.0f;
		a=0.0f;
	}

	GL4f(float x_old,float y_old, float z_old, float a_old){
		x=x_old;
		y=y_old;
		z=z_old;
		a=a_old;
	}

	GL4f operator+(GL4f temp){
		return GL4f(temp.x + x, temp.y + y, temp.z + z, temp.a + a);
	}
	
	GL4f operator-(GL4f temp){
		return GL4f(temp.x - x, temp.y - y, temp.z - z, temp.a - a);
	}

	GL4f operator*(float num){
		return GL4f(x * num, y * num, z * num, a * num);
	}

	GL4f operator/(float num){
		return GL4f(x / num, y / num, z / num, a / num);
	}
};


//Loads 4 integers
struct GL4i{
	int x;
	int y;
	int z;
	int a;

	GL4i(){
		x=0;
		y=0;
		z=0;
		a=0;
	}
	
	GL4i(int x_old,int y_old, int z_old, int a_old){
		x=x_old;
		y=y_old;
		z=z_old;
		a=a_old;
	}

	GL4i operator+(GL4i temp){
		return GL4i(temp.x + x, temp.y + y, temp.z + z, temp.a + a);
	}
	
	GL4i operator-(GL4i temp){
		return GL4i(temp.x - x, temp.y - y, temp.z - z, temp.a - a);
	}

	GL4i operator*(int num){
		return GL4i(x * num, y * num, z * num, a * num);
	}

	GL4i operator/(int num){
		return GL4i(x / num, y / num, z / num, a / num);
	}
};

//Loads boundings
struct GLBounding{
private:
	float xmin;
	float xmax;
	float ymin;
	float ymax;
	float zmin;
	float zmax;
	
public:

	GLBounding(){
		clear();
	}


	GLBounding(float *pointsArray,int numPoints){
		clear();
		addBounding(pointsArray,numPoints);
	}


	GLBounding(GL3f *pointsArray,int numPoints){
		clear();
		addBounding(pointsArray,numPoints);
	}

	GLBounding(const GLBounding &bounding){
		clear();
		addBounding(bounding);
	}

	GLBounding (float xmin_old,float xmax_old,float ymin_old,float ymax_old,float zmin_old,float zmax_old){
		xmin=xmin_old;
		xmax=xmax_old;
		ymin=ymin_old;
		ymax=ymax_old;
		zmin=zmin_old;
		zmax=zmax_old;
	}

	float getXmin(){
		return xmin;
	}

	float getXmax(){
		return xmax;
	}

	float getYmin(){
		return ymin;
	}

	float getYmax(){
		return ymax;
	}

	float getZmin(){
		return zmin;
	}

	float getZmax(){
		return zmax;
	}

	void addBounding(float *pointsArray,int numPoints){
		for (register int k=0;k<numPoints*3;k+=3){
			if (xmax<pointsArray[k]) xmax=pointsArray[k];
			if (xmin>pointsArray[k]) xmin=pointsArray[k];
			if (ymax<pointsArray[k+1]) ymax=pointsArray[k+1];
			if (ymin>pointsArray[k+1]) ymin=pointsArray[k+1];
			if (zmax<pointsArray[k+2]) zmax=pointsArray[k+2];
			if (zmin>pointsArray[k+2]) zmin=pointsArray[k+2];
		}

	}

	void addBounding(const GLBounding &bounding){
		if (bounding.xmin<xmin ) xmin=bounding.xmin;
		if (bounding.ymin<ymin ) ymin=bounding.ymin;
		if (bounding.zmin<zmin ) zmin=bounding.zmin;
		if (bounding.xmax>xmax ) xmax=bounding.xmax;
		if (bounding.ymax>ymax ) ymax=bounding.ymax;
		if (bounding.zmax>zmax ) zmax=bounding.zmax;
	}

	void addBounding(GL3f *pointsArray,int numPoints){
			
		for (register int k=0;k<numPoints;k++){
			if (xmax<pointsArray[k].x) xmax=pointsArray[k].x;
			if (xmin>pointsArray[k].x) xmin=pointsArray[k].x;
			if (ymax<pointsArray[k].y) ymax=pointsArray[k].y;
			if (ymin>pointsArray[k].y) ymin=pointsArray[k].y;
			if (zmax<pointsArray[k].z) zmax=pointsArray[k].z;
			if (zmin>pointsArray[k].z) zmin=pointsArray[k].z;
		}
	}

	void clear(){
		xmin=500000.0f;
		xmax=-500000.0f;
		ymin=500000.0f;
		ymax=-500000.0f;
		zmin=500000.0f;
		zmax=-500000.0f;
	}

	GLBounding operator+(GLBounding &temp){
		return GLBounding(temp.xmin + xmin, temp.xmax + xmax, 
				  temp.ymin + ymin, temp.ymax + ymax,
				  temp.zmin + zmin, temp.zmax + zmax);
	}

	
	GLBounding operator-(GLBounding &temp){
		return GLBounding(temp.xmin - xmin, temp.xmax - xmax, temp.ymin - ymin, temp.ymax - ymax, temp.zmin - zmin, temp.zmax - zmax);
	}

	GLBounding operator+(GL3f temp){
		return GLBounding(temp.x + xmin, temp.x + xmax, temp.y + ymin, temp.y + ymax, temp.z + zmin, temp.z + zmax);
	}
	
	GLBounding operator-(GL3f temp){
		return GLBounding(temp.x - xmin, temp.x - xmax, temp.y - ymin, temp.y - ymax, temp.z - zmin, temp.z - zmax);
	}

	GLBounding operator*(int num){
		return GLBounding(xmin * num, xmax * num, ymin * num, ymax * num, zmin * num, zmax * num);
	}

	GLBounding operator/(int num){
		return GLBounding(xmin / num, xmax / num, ymin / num, ymax / num, zmin / num, zmax / num);
	}

	void print(){
		printf("xmin: %f xmax: %f ymin: %f ymax: %f zmin: %f zmax: %f\n",xmin,xmax,ymin,ymax,zmin,zmax);
	}

};

void glhTranslatef(GL3f posicion);

void glhRotatef(GL3f rotation);

void glhScalef(GL3f scale);

void glhVertex3f(GL3f vertex);

void gluhLookAt(GL3f pos, GL3f obj, float x, float y, float z);

//Le pasas un bounding y te devuelve el centro de este
GL3f getBoundingPos(GLBounding bounding);

//Le pasas la "x" y la "y" y te pasa el mundo a 2d
void glEnter2D(int width, int height);

//Cierra el mundo el 2d
void glExit2D();

//Pinta un texto por pantalla, le pasas el texto, la x, la y y la fuente.
void drawString(char *messageStr, int x, int y,char* message_font);

void drawImage(unsigned int tid,float z);
void drawImage(unsigned int tid);

void glhRandomize();


#endif
