#include "ParticleSystem.h"

ParticleSystem::ParticleSystem()
{
	init();
}

ParticleSystem::ParticleSystem(char *file)
{
	init();

	chdir(getDirs(file));
	FILE *fd = fopen(file,"r");
	if (fd == NULL)
	{
		printf("Cannot find '%s' particle system.\n",file);
		return;
	}

	char buf[1024],key[1024];

	while(true)
	{
		fgets(buf,1000,fd);
		if (feof(fd)) break;

		sscanf(buf,"%s",key);
		if (key[0]=='#') continue;

		if (!strcmp(key,"POSITION"))
		{
			sscanf(buf,"%s %f %f %f",key,&x,&y,&z);
		} else
		if (!strcmp(key,"SPEED"))
		{
			sscanf(buf,"%s %f %f %f",key,&vx,&vy,&vz);
		} else
		if (!strcmp(key,"SPEED-RANGE"))
		{
			float x,y,z;
			sscanf(buf,"%s %f %f %f",key,&x,&y,&z);
			rx = (int) (x * 1000);
			ry = (int) (y * 1000);
			rz = (int) (z * 1000);
		} else
		if (!strcmp(key,"GRAVITY"))
		{
			sscanf(buf,"%s %f",key,&gravity);
		} else
		if (!strcmp(key,"COLOR"))
		{
			sscanf(buf,"%s %f %f %f",key,&r,&g,&b);
		} else
		if (!strcmp(key,"TTL"))
		{
			sscanf(buf,"%s %d",key,&ttl);
		} else
		if (!strcmp(key,"TEXTURE"))
		{
			char file[1024];
			sscanf(buf,"%s %s",key,file);
			this->setTexture(file);
		} else
		if (!strcmp(key,"DELAY"))
		{
			sscanf(buf,"%s %d",key,&delay);
		} else
		if (!strcmp(key,"NUM"))
		{
			int num;
			sscanf(buf,"%s %d",key,&num);
			this->setNumberOfParticles(num);
		} else
		if (!strcmp(key,"SIZE"))
		{
			sscanf(buf,"%s %f",key,&size);
		} else
		if (!strcmp(key,"GROW"))
		{
			sscanf(buf,"%s %f",key,&grow);
		} else
		if (!strcmp(key,"WIND"))
		{
			sscanf(buf,"%s %f %f %f",key,&wx,&wy,&wz);
		} else
		if (!strcmp(key,"WIND-RANGE"))
		{
			float x,y,z;
			sscanf(buf,"%s %f %f %f",key,&x,&y,&z);
			wrx = (int) (x*1000);
			wry = (int) (y*1000);
			wrz = (int) (z*1000);
		} else
		if (!strcmp(key,"SHAPE"))
		{
			char shape[1024];
			sscanf(buf,"%s %s",key,shape);
			if (!strcmp(shape,"LINES"))
				this->shape = LINES;
			else
			if (!strcmp(shape,"QUADS"))
				this->shape = QUADS;
			else
			if (!strcmp(shape,"TRIANGLES"))
				this->shape = TRIANGLES;
			else
				printf("INVALID SHAPE TYPE: '%s'\n",shape);
		}
	}

	fclose(fd);
}

ParticleSystem::~ParticleSystem()
{
	delete[] p;
	if (texture)
	TGAloader_untid(texture);
}

/* <CONTROL> */

void ParticleSystem::init()
{
	running = false;
	n = 0;
	w = h = 0;
	ttl = 0;
	x = y = z = 0;
	rx = ry = rz = 0;
	vx = vy = vz = 0;
	wx = wy = wz = 0;
	wrx= wry= wrz= 0;
	last = SDL_GetTicks();
	delay = 10; /* 10ms */
	grow = 0;
	size = 0.02;
	texture = 0;
	shape = TRIANGLES;
	glhRandomize();
}

void ParticleSystem::start()
{
	running = true;
}

void ParticleSystem::stop()
{
	running = false;
}

void ParticleSystem::update()
{
	int i;

	if (running)
	{
	unsigned int now = SDL_GetTicks(); 

	if ( last+delay < now)
	{
		last = now;
		for (i=0;i<n;i++)
		{
		/* update time to live */
		p[i]->ttl++;
		if ( p[i]->ttl > ttl )
		{
			p[i]->ttl = 0;
			/* position */
			p[i]->x = x;
			p[i]->y = y;
			p[i]->z = z;
			/* color */
			p[i]->r = r;
			p[i]->g = g;
			p[i]->b = b;
			/* size */
			p[i]->size = size;
			/* speed */
			float m;
			if (rx) m = rand()%rx; else m=0;
			p[i]->vx = vx+((float)(m)/1000);
			if (ry) m = rand()%ry; else m=0;
			p[i]->vy = vy+((float)(m)/1000);
			if (rz) m = rand()%rz; else m=0;
			p[i]->vz = vz+((float)(m)/1000);
		} else {
			p[i]->size += grow;

			float m;
			if (wrx) m = rand()%wrx; else m=0;
			p[i]->x += p[i]->vx + wx+((float)(m)/1000);
			if (wrx) m = rand()%wry; else m=0;
			p[i]->y += p[i]->vy + wy+((float)(m)/1000);
			if (wrx) m = rand()%wrz; else m=0;
			p[i]->z += p[i]->vz + wz+((float)(m)/1000);

			p[i]->vy-= gravity;
		}
		}
	}
	}
}

void ParticleSystem::draw(float yaw,float yow)
{
	float SIN=sin(yaw);
	float COS=cos(yaw);

	glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D,texture);
	for (int i=0;i<n;i++)
	{
        glPushMatrix();
        
        glColor3f
	(
		p[i]->r,
		p[i]->g,
		p[i]->b
	);

        glTranslatef
	(
		p[i]->x,
		p[i]->y,
		p[i]->z
	);

	float s = p[i]->size;

        switch(shape)
        {
        case TRIANGLES:
                glBegin(GL_TRIANGLES);
		glVertex3f(-SIN*s,0,COS*s);
		glVertex3f(0,s,0);
		glVertex3f(-SIN*-s,0,COS*-s);
                glEnd();
                break;
        case QUADS:
		// TODO paralel to screen
                glBegin(GL_QUADS);
                glVertex3f( s,    0, 0);
                glVertex3f(    0, s, 0);
                glVertex3f(-s,    0, 0);
                glVertex3f(-s,    0, 0);
                glEnd();
                break;
        case LINES:
                break;
        }
        
        glPopMatrix();
	}
	//glEnable(GL_LIGHTING);
}

/* </CONTROL> */

void ParticleSystem::
setNumberOfParticles(int num)
{
	if (n == 0)
	{
		n = num;

		p = new Particle*[n];
		for (int i=0;i<n;i++)
		p[i] = new Particle();
	}
}

void ParticleSystem::
setPosition(float x,float y, float z)
{
	this->x=x;
	this->y=y;
	this->z=z;
}

void ParticleSystem::
setDirection(GL3f d)
{
}

void ParticleSystem::
setGravity(float f)
{
	gravity = f;
}

void ParticleSystem::
setGravityDirection(GL3f d)
{
}

void ParticleSystem::
setSurfaceWidth(float w)
{
}

void ParticleSystem::
setSurfaceHeight(float h)
{
}

void ParticleSystem::
setTexture(char *file)
{
	texture = TGAloader_tid(file);
}

void ParticleSystem::
setTTL(unsigned int t)
{
	this->ttl = t;
}

void ParticleSystem::
setDelay(unsigned int d)
{
	this->delay = d;
}

void ParticleSystem::
setSize(float s)
{
	this->size = s;
}

void ParticleSystem::
setGrowage(float g)
{
	this->grow = g;
}

void ParticleSystem:: setWind(
	float wx, float wrx,
	float wy, float wry,
	float wz, float wrz)
{
	/* wind range (random) */
	this->wx = wx;
	this->wy = wy;
	this->wz = wz;
	/* base wind */
	this->wrx = (int)(wrx*1000);
	this->wry = (int)(wry*1000);
	this->wrz = (int)(wrz*1000);
}

void ParticleSystem::
setBase(float w,float h)
{
	this->w = w;
	this->h = h;
}

void ParticleSystem::setSpeed(
	float vx, float x,
	float vy, float y,
	float vz, float z)
{
	/* base speed  #### speed range */
	this->vx = vx; /**/ this->rx = (int)(x*1000);
	this->vy = vy; /**/ this->ry = (int)(y*1000);
	this->vz = vz; /**/ this->rz = (int)(z*1000);
}
