#include "Properties.h"

Properties::Properties(char *file)
{
	//object3dPointer = false;
	object3dObj = new Object3D(file);
	position = GL3f(0.0f,0.0f,0.0f);

	init();
}

Properties::Properties (char *file, float posX, float posY, float posZ)
{
	//object3dPointer = false;
	object3dObj = new Object3D(file);
	position = GL3f(posX,posY,posZ);

	init();
}

Properties::Properties(Object3D *obj)
{
	//object3dPointer = true;
	object3dObj = obj;
	position = GL3f(0.0f,0.0f,0.0f);

	init();
}

Properties::Properties (Object3D *obj, float posX, float posY, float posZ)
{
	//object3dPointer = true;
	object3dObj = obj;
	position = GL3f(posX,posY,posZ);

	init();
}

Properties::Properties(const Properties &p)
{
	object3dObj = p.object3dObj;
	object3dPointer = true;
	position = p.position;
	rotationAngle = p.rotationAngle;
	rotation = p.rotation;
	rotationSpeed = p.rotationSpeed;
	final = p.final;
	rotateTimeLeft = p.rotateTimeLeft;
	numAnimacio = 0;
	nextAnimacio = 0;
	posAnimacio = -1;
	speed = p.speed;
	visible = p.visible;
#if _PVS
	id = p.id;
#endif
	numDFaces=object3dObj->getNumFaces();
	nextDesintegration = 0;
	nextExplosion = 0;
	explodeVertex = 0;
	explode = false;
	explosionRang = 100.0f;
	desintegracion = 0;
}

void Properties::init()
{
	explosionRang = 100.0f;
	explodeVertex = 0;
	nextExplosion = 0;
	explode = false;
	desintegracion = 0;
	nextDesintegration = 0;
	numDFaces = object3dObj->getNumFaces();
	visible = true;
	final = SDL_GetTicks();
	numAnimacio = 0;
	nextAnimacio = 0;
	posAnimacio = -1; //Lo primero que hace el programa es aumentarla a 1
	rotateTimeLeft = 0;
	explode = false;

	//La velocidad inicial de animacion del modelo es la que viene por defecto en el object3d
	speed = object3dObj->getSpeedDefault(); 
	rotation = 0;
	rotationAngle = 1;
	rotationSpeed = 0;
#if _PVS
	id = 0;
#endif
	visible = true;
}

Properties::~Properties()
{
	//Si el object3d no ha sido creado en la clase no lo destruimos y dejamos
	//que lo haga su creador
	delete object3dObj;
	//if (!object3dPointer)
	//	delete object3dObj;
}

void Properties::update()
{
	if (explode)
	{
		if (SDL_GetTicks()>=nextExplosion)
		{
			nextExplosion=SDL_GetTicks()+EXPLOSION_DELAY;

			if(explosionRang>1.0f) 
			{
				explosionRang-=(explosionRang/4.0f);
			} else {
				explode=false;
				if (!desintegracion) visible=false;
			}

			GL3f normal;
			GL3f triangle[3];
			float *_vertex=object3dObj->getVertex(posAnimacio);

			if (!explodeVertex)
				explodeVertex = new float[object3dObj->getNumFaces()*9];

			for (register int i=0; i<object3dObj->getNumFaces()*9;i+=9)
			{
				int j=0;

				for (j=0;j<3;j++)
				{
					triangle[j].x=_vertex[i+(j*3)];
					triangle[j].y=_vertex[i+(j*3)+1];
					triangle[j].z=_vertex[i+(j*3)+2];
				}

				normal = Normal(triangle);
				normal = Normalize(normal);
				normal = normal/explosionRang;
				
				for (j=0;j<3;j++)
					triangle[j]=triangle[j]+(normal);

				for (j=0; j< 9; j+=3)
				{
					explodeVertex[j+i]=triangle[j/3].x;
					explodeVertex[j+i+1]=triangle[j/3].y;
					explodeVertex[j+i+2]=triangle[j/3].z;
				}
			}
		}
	}
	if (desintegracion)
	{
		if (SDL_GetTicks()>=nextDesintegration)
		{
			if (desintegracion>0 && numDFaces>=0 && numDFaces<object3dObj->getNumFaces())
			{
				nextDesintegration=SDL_GetTicks()+DESINTEGRATION_DELAY;
				numDFaces=numDFaces+1;
				if (numDFaces==object3dObj->getNumFaces()) desintegracion=0;
			}
			else if (desintegracion<0 && numDFaces>0 && numDFaces<=object3dObj->getNumFaces())
			{
				nextDesintegration=SDL_GetTicks()+DESINTEGRATION_DELAY;
				numDFaces=numDFaces-1;
			}
		}
	}
	
	//Cuando llega el momento del cambio de animacion le indicas el nuevo momento en que
	//tendr que hacerlo
	if (SDL_GetTicks()>=final && !desintegracion && !explode)
	{
		final=SDL_GetTicks()+speed;
	//	printf("->%d %d %d\n",posAnimacio,object3dObj->getTipoAnimacio(numAnimacio),object3dObj->getFirstFrame(numAnimacio+1));
		posAnimacio++;
		//Si el frame actual de la animacion es igual al primer frame de la animacion 
		//siguiente significa que nos estamos saliendo de nuestra animacion
		if (posAnimacio==object3dObj->getFirstFrame(numAnimacio+1))
		{
			//Si la animacion es del tipo bucle, volvemos al primer frame de esta.
			//En caso contrario volvemos al frame anterior
			if (nextAnimacio!=numAnimacio)
			{
				numAnimacio=nextAnimacio;
				posAnimacio=object3dObj->getFirstFrame(numAnimacio);
			}
			else if (!object3dObj->getTipoAnimacio(numAnimacio)) posAnimacio=object3dObj->getFirstFrame(numAnimacio);
			else posAnimacio--;
		}
	 
	}

	//Si la velocidad de rotacion es mayor a 0 significa que el objeto tiene rotacion propia
	//y por lo tanto li giramos a esa velocidad
	if (rotationSpeed>0 && !explode)
	{
		if (SDL_GetTicks()>=rotateTimeLeft)
		{
			rotateTimeLeft=SDL_GetTicks()+rotationSpeed;
			setRotation(getRotation()+rotationAngle);

		}
	}	
		
}

int Properties::setAnimation(int numAnimacio_x,int inmediate, bool changeAnyWay)
{
	if (numAnimacio_x==numAnimacio && !changeAnyWay)
		return 1;

	if (numAnimacio_x>=0 || numAnimacio_x<object3dObj->getAnimacioTotal())
	{
		nextAnimacio=numAnimacio_x;
		if (inmediate>0)
		{
			numAnimacio=numAnimacio_x;
			//Le digo que el siguiente frame a dibujar es el primero de la nueva animacin
			posAnimacio=object3dObj->getFirstFrame(numAnimacio);
		}
	}
	else return 0;

	/* Animation changed properly */
	return 1;
}


int Properties::getNumAnimacio()
{
	return numAnimacio;
}

int Properties::getPosAnimacio()
{
	return posAnimacio;
}

int Properties::getSpeed()
{
	return speed;
}

void Properties::setSpeed(int speed_copia)
{
	if (speed_copia<20) speed=20;
	else if (speed_copia>1500) speed=1500;
	else speed=speed_copia;
}

GL3f Properties::getPos()
{
	return position; 
}

void Properties::setPos(GL3f pos)
{
	position=pos;
 }

float Properties::getRotation()
{
	return rotation; 
}

float Properties::getRotationAngle()
{
	return rotationAngle;
}

void Properties::setPos(float posX, float posY, float posZ)
{ 
	position.x=posX;
	position.y=posY;
	position.z=posZ;
}

void Properties::setRotation(float rot)
{
	if (rot>360.0f)
		rot -= 360.0f;
	else
	if (rot<0.0f)
		rot += 360.0f;

	rotation=rot;
}

void Properties::setRotationAngle (float rot)
{
	rotationAngle=rot;
}

void Properties::activateRotation(int rot)
{
	rotationSpeed = rot;
	rotateTimeLeft = rot+SDL_GetTicks();
}

void Properties::setObject3D (Object3D *obj)
{
	init();

	object3dObj = obj;
}

Object3D *Properties::getObject3D()
{
	 return object3dObj;
}

void Properties::setVisible(bool mode)
{
	desintegracion=0;
	(mode)?numDFaces=object3dObj->getNumFaces():numDFaces=0;
	visible=mode;	
}


bool Properties::isVisible()
{
	 return visible;
}

#if _PVS
void Properties::setId(int id_)
{
	id=id_;
}
#endif

int Properties::getId()
{
	return id;
}

void Properties::setDesintegration(int mode)
{
	 desintegracion=mode;
	 numDFaces=object3dObj->getNumFaces();
	 visible=true;
}

int Properties::isDesintegrating()
{
	return desintegracion;
}

int Properties::getNumDFaces()
{
	return numDFaces;
}

void Properties::setExplosion(bool mode)
{
	explode=mode;
	visible=true;
	explosionRang=100.0f;
}

bool Properties::isExploting()
{
	return explode;
}

float *Properties::getExplotingVertex()
{
	return explodeVertex;
}



