#include "Render.h"

extern SDL_Surface *WINDOW;

Render::Render()
{
	step=0.0f; // walk effect
	nextChange=0;
	viewMap=false; // not needed, but useful
	modo=1;
}

Render::~Render()
{
	// TODO destroy Render class...any?
}


void Render::setViewMap(bool b)
{
	viewMap=b;
}

void Render::setTexture(GLuint text)
{
	glBindTexture(GL_TEXTURE_2D,text);
}

void Render::draw(Stage *fase)
{
	int *PVS = 0;

	glEnable(GL_SCISSOR_TEST); // allow two viewports
	glMatrixMode(GL_MODELVIEW); // yet defined !?

	/* FOG support */
	fase->beginFog();
	if (LIGHT)
	fase->lights->enable();

	// Entire screen
	glViewport(0,0,WIDTH,HEIGHT);
	glScissor(0,0,WIDTH,HEIGHT);

	//Limpiamos la pantalla
	glClearColor(1,1,1,1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//Cojemos el PVS de la habitacion en la que estamos
#if _PVS
	PVS=fase->getPVS(fase->getCamaraId());
#endif

	//Reseteamos el glLineWidth a 2
	glLineWidth(2.0f); // here??


	//Miramos que en que tipo de debug estamos
	switch(DEBUG)
	{
	case SOLO_CARAS:
		glPolygonMode(GL_FRONT, GL_FILL);
		modo=1;
		break;
	case SOLO_VERTICES:
		glPolygonMode(GL_FRONT, GL_LINE);
		modo=1;
		break;
	case CARAS_Y_VERTICES:
		glPolygonMode(GL_FRONT, GL_LINE);
		modo=2;
		break;
	}

	//Activamos la textura o no dependiendo del modo en que estamos
	if (TEXTURA && modo==1) glEnable(GL_TEXTURE_2D);
	else glDisable(GL_TEXTURE_2D);

	//Nos colocamos en el mundo y empezamos a mirar
	drawCamera(fase);

	for (;modo>0;modo--)
	{
		pintaEscenario(fase,PVS);
		pintaItems(fase,PVS);
		if (fase->isThirdView()) pintaEnemigos(fase,PVS);
			
		//Modo 2 se usa en modo CARAS_Y_VERTICES.
		//Cuando esta activado pinta las linias que forman los triangulos
		//Una vez hecho pasa a modo 1 y pinta normal
		if (modo==2)
		{
			glPolygonMode(GL_FRONT, GL_FILL);
			if (TEXTURA) glEnable(GL_TEXTURE_2D);
		}
	}

	if (BOUNDING) pintaBoundings(fase,PVS);
#if _PVS
	if (PORTALES) pintaPortales(fase,PVS);
#endif

	if (DEBUG_COLLISION)
	fase->mapa->drawCollision();

	if (DRAW_TRACES) fase->trace->draw();

	fase->mapa->drawParticleSystems(fase->getRotation(),0);

	fase->mapa->endFog();
	fase->lights->disable();

	/* SKY BOX */
	fase->getMap()->getSky()->draw(fase->getCamaraPos());
	fase->getMessageControl()->drawMessages();
	fase->getCamaraInterface()->paint();


	/* VIEWPORT MAP */
	if (viewMap)
	{
	if (FOG)
	glDisable(GL_FOG);

	glViewport(WIDTH-(WIDTH/6)-2,HEIGHT-(HEIGHT/4)-2,WIDTH/6+4,HEIGHT/4+4);
	glScissor(WIDTH-(WIDTH/6)-2,HEIGHT-(HEIGHT/4)-2,WIDTH/6+4,HEIGHT/4+4);
	glClearColor(0.6,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);

	glViewport(WIDTH-(WIDTH/6),HEIGHT-(HEIGHT/4),WIDTH/6,HEIGHT/4);
	glScissor(WIDTH-(WIDTH/6),HEIGHT-(HEIGHT/4),WIDTH/6,HEIGHT/4);
	glMatrixMode(GL_PROJECTION);

	glLoadIdentity();

	glOrtho(-3.0, 1.5, -3.0, 1.5, -100.0, 100.0);
	glMatrixMode(GL_MODELVIEW);

	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);
	glColor3f(1,1,1);

	glEnable(GL_TEXTURE_2D); // must be enabled on each VP

	/* draw world */
	GL3f pos=fase->getCamaraPos();
	GL3f obj=fase->getCamaraObj();

	gluLookAt(
		pos.x-1,pos.y+PLAYER_HEIGHT+40,pos.z,
		pos.x-1,pos.y,pos.z,
		0,0,1);

	glColor3f(1,0,0);
	glBegin(GL_QUADS);
		glVertex3f(pos.x,    pos.y+30,pos.z);
		glVertex3f(pos.x-0.5,pos.y+30,pos.z);
		glVertex3f(pos.x-0.5,pos.y+30,pos.z-0.5);
		glVertex3f(pos.x,    pos.y+30,pos.z-0.5);
	glEnd();

	glColor3f(1,1,1);
	// X = 
	glPushMatrix();
		glTranslatef(pos.x,pos.y,pos.z);
		glRotatef(270+fase->getRotation()*180/PI,0,1,0);
		glTranslatef(-pos.x,-pos.y,-pos.z);

		pintaEscenario(fase,PVS);
		pintaItems(fase,PVS);
		pintaEnemigos(fase,PVS);
	glPopMatrix();

	glColor3f(1,0,0);
	glBegin(GL_TRIANGLES);
		glVertex3f(pos.x,pos.y+2,pos.z+0.2);
		glVertex3f(pos.x+0.2,pos.y+2,pos.z);
		glVertex3f(pos.x-0.2,pos.y+2,pos.z);
	glEnd();

	glFlush();
	if (FOG)
	glEnable(GL_FOG);
	}

#if _PVS
	delete[]PVS;
#endif

}

void Render::setWalkEffect(float f)
{
	step=f;
}

void Render::drawCamera(Stage *fase)
{
	GL3f pos=fase->getCamaraPos();
	GL3f obj=fase->getCamaraObj();
	GL2f yaw=fase->getCamaraYaw();

#if SOUND
	fase->getMap()->playSound(fase->getCamaraId(),RUNNING,pos);
#endif

	if (fase->isThirdView())
	{
	/* 3rd person view */
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluPerspective(FOV,WIDTH/HEIGHT,0.1,3000.0);  
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	gluLookAt(
		pos.x-(obj.x-pos.x),   // X
		pos.y+PLAYER_HEIGHT*2, // Y
		pos.z-(obj.z-pos.z),   // Z
		obj.x,obj.y+PLAYER_HEIGHT,obj.z,
		0,1.0f,0);

	// todo DRAW CAMERA
	drawEnemy((Enemy*)fase->getMyBody());
	glPopMatrix();
	} else {

	/* 1st person view */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(FOV,WIDTH/HEIGHT,0.1,3000.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	gluLookAt(
		pos.x,pos.y+PLAYER_HEIGHT,pos.z,
		obj.x,obj.y+PLAYER_HEIGHT,obj.z,
		step*0.08,1.0f,step*STEP); // XXX add walk effect x)

	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	Properties *camara=fase->getCamFirstView();

	glTranslatef(0.0f,-0.05f,-0.1f);

	// draw weapon
	drawObject (	camara->getObject3D()->getTexture(),
			camara->getObject3D()->getVertex(camara->getPosAnimacio()),
			camara->getObject3D()->getTextureVertex(),
			camara->getObject3D()->getNumFaces());

	glPopMatrix();
	}
}

void Render::pintaEscenario(Stage *fase, int *PVS)
{
	/* walk on all rooms */
	for (int j=0;j<fase->getMap()->getNumRooms();j++)
	{
		Room *hab = fase->getMap()->getRoom(j);

		/* is the target room watchable? */
		if(*PVS && PVS[hab->getId()])
		{
			beginVertexArray(hab->getVertex(),hab->getTextureVertex());
						
			for (int i=0; i<hab->getNumTextures();i++)
			{
			if (LIGHT)
			fase->getMap()->useMaterial(hab->getTextureNumber(i));

			drawVertexArray
				(
				fase->getMap()->getTexture(hab->getTextureNumber(i)),
				hab->getNormals(),
				hab->getFirstTexture(i)/3, 
				hab->getFirstTexture(i+1)/3
				);
			}

			endVertexArray();
		}
	}
}

void Render::pintaItems(Stage *fase, int *PVS)
{
	int i=0;
	int frame;

	for (i=0;i<fase->getNumItems();i++)
	{
		Properties *item=fase->getItem(i);
		/* item watchable from our room? */
		if (*PVS && PVS[item->getId()])
		if (item->isVisible()) 
		{
			/* take the actual frame of animation */
			frame=item->getPosAnimacio();

		 	setTexture(item->getObject3D()->getTexture());

			GL3f positionObj = item->getPos();
			glPushMatrix();

			/* move and rotate */
			glhTranslatef(positionObj);
			glRotatef(item->getRotation(),0,1,0);
			
			/* to draw! */
			if (item->isExploting())
			{
			drawObject(     item->getObject3D()->getTexture(),
					item->getExplotingVertex(),
					item->getObject3D()->getTextureVertex(),
					item->getObject3D()->getNumFaces() );
			} else {
				if (item->isDesintegrating())
				{
				drawObject(
					item->getObject3D()->getTexture(),
					item->getObject3D()->getVertex(frame),
					item->getObject3D()->getTextureVertex(),
					item->getNumDFaces() );
				drawObject(
					item->getObject3D()->getTexture(),
					item->getObject3D()->getVertex(frame),
					item->getObject3D()->getTextureVertex(),
					item->getObject3D()->getNumFaces(),0.5f);
				} else {
				drawObject(
					item->getObject3D()->getTexture(),
					item->getObject3D()->getVertex(frame),
					item->getObject3D()->getTextureVertex(),
					item->getObject3D()->getNumFaces() );
				}
			glPopMatrix();
			}
		}
	}
}

void Render::drawEnemy(Enemy *enemigo)
{
	register int j;

	glPushMatrix();

	/* head, body, legs */
	for (j=0;j<3;j++)
	{
		/* get current animation frame */
		int frame=enemigo->getControl(j)->getPosAnimacio();

		/* set texture */
		GL3f positionObj=enemigo->getControl(j)->getPos();

		/* move and rotate */
		glhTranslatef(enemigo->getUnion(j));
		if (!j)	glRotatef(-enemigo->getRotation()-enemigo->getBaseAngle(),0,1,0);

		/* draw it */
		if (enemigo->isDying())
		{
		drawObject(
			enemigo->getControl(j)->getObject3D()->getTexture(),
			enemigo->getControl(j)->getObject3D()->getVertex(frame),
			enemigo->getControl(j)->getObject3D()->getTextureVertex(),
			enemigo->getControl(j)->getNumDFaces(),0.5f);

		drawObject(
			enemigo->getControl(j)->getObject3D()->getTexture(),
			enemigo->getControl(j)->getExplotingVertex(),
			enemigo->getControl(j)->getObject3D()->getTextureVertex(),
			enemigo->getControl(j)->getObject3D()->getNumFaces());
		} else {
		drawObject(
			enemigo->getControl(j)->getObject3D()->getTexture(),
			enemigo->getControl(j)->getObject3D()->getVertex(frame),
			enemigo->getControl(j)->getObject3D()->getTextureVertex(),
			enemigo->getControl(j)->getNumDFaces());
		}
	}

	//int frame=enemigo->getControl(BODY)->getPosAnimacio();

	//Le aplicamos la textura que necesite
#if 0
	glhTranslatef(enemigo->getUnion(LEGS));
	glRotatef(-enemigo->getRotation()-90,0,1,0);
	glhTranslatef(enemigo->getUnion(BODY));
	glhTranslatef(enemigo->getUnion(WEAPON_POS));
				
/* XXX add 3person weapon */
//#if 0
	glRotatef(enemigo->getWeaponRot().x,0,1,0);
	if (enemigo->haveWeapon())
	{
	int weaponFrame=enemigo->getWeapon()->getThirdView()->getPosAnimacio();
	glRotatef(enemigo->getWeaponRot().y,1,0,0);
	//printf("%f %f\n",enemigo->getWeaponRot().x,enemigo->getWeaponRot().y);
	//	enemigo->getUnion(WEAPON_POS).print();
	drawObject(
		enemigo->getWeapon()->getThirdView()->getObject3D()->getTexture(),
		enemigo->getWeapon()->getThirdView()->getObject3D()->getVertex(weaponFrame),
		enemigo->getWeapon()->getThirdView()->getObject3D()->getTextureVertex(),
		enemigo->getWeapon()->getThirdView()->getNumDFaces());
	}
#endif
	glPopMatrix();
}

void Render::pintaEnemigos(Stage *fase, int *PVS)
{
	for (int i=0;i<fase->getMaxPlayers();i++)
	{
		Enemy *enemigo=fase->getEnemy(i);
		
		if (enemigo)
		{
#if _PVS
		if (PVS[enemigo->getId()]) 
#endif
		if (enemigo->isVisible())
				this->drawEnemy(enemigo);
		}
	}

}

#if _PVS
void Render::pintaPortales(Stage *fase, int *PVS)
{
	//Puntero temporal
	GL3f *vertex;

	//Si solo pintamos vertices hacemos las linias mas gruesas para que se vean mas 
	if (DEBUG==SOLO_VERTICES) glLineWidth(10.0f);

	//Si no pintamos texturas, las activamos para hacer las transparencias correctamente
	if (!TEXTURA) glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glColor4f(0.0f,1.0f,1.0f,0.5f);
	glBegin(GL_QUADS);
//	printf("habitacion: %d numPortals: %d\n",fase->getCamaraId(),fase->getMap()->getHabitacion(fase->getCamaraId())->getNumPortals());
	//Cojemos todos los portales de nuestra habitacion y los pintamos
	int nPortals = fase->getMap()->
		getRoom(fase->getCamaraId())->getNumPortals();

	for (int cont=0; cont<nPortals; cont++)
	{
		vertex=fase->getMap()->getRoom(fase->getCamaraId())->getPortalVertex(cont);
						
		glhVertex3f(vertex[1]);
		glhVertex3f(vertex[3]);
		glhVertex3f(vertex[2]);
		glhVertex3f(vertex[0]);				
	}

	glPopMatrix();
	glEnd();
	glDisable(GL_BLEND);

	//Si no pintamos texturas las deshabilitamos
	if (!TEXTURA) glDisable(GL_TEXTURE_2D);

	//Reseteamos el grosor de la linia a 2
	if (DEBUG==SOLO_VERTICES) glLineWidth(2.0f);
}
#endif

void Render::pintaBoundings(Stage *fase, int *PVS)
{
	int cont=0;

	// enemies boundings
	for (cont=0;cont<fase->getActualPlayers();cont++)
	{
		Enemy *enemigo=fase->getEnemy(cont);
#if _PVS
		if (PVS[enemigo->getId()]){
#endif
			glPushMatrix();
			glhTranslatef(enemigo->getPos());
			drawBoundingCube(enemigo->getBounding());			
			glPopMatrix();
#if _PVS
		}
#endif
	}

	// item boundings
	for (cont=0;cont<fase->getNumItems();cont++)
	{
		Properties *item_tempo=fase->getItem(cont);
#if _PVS
		if (PVS[item_tempo->getId()]){
#endif
			glPushMatrix();
			glhTranslatef(item_tempo->getPos());
			drawBoundingCube(item_tempo->getObject3D()->getBounding());
			glPopMatrix();
#if _PVS
		}
#endif
	}

}

void Render::beginVertexArray(float *vertexArray, float *textureArray)
{
	glEnableClientState(GL_VERTEX_ARRAY); 
	glVertexPointer(3,GL_FLOAT,0,vertexArray); 

	//Si pintamos texturas le pasamos el array de texturas al glTexCoordPointer.
	//Si no las pintamos, le pasamos el array de vertices al glColorPointer para conseguir
	//un colorido paranoico
	if (TEXTURA && modo!=2){
		glEnableClientState(GL_TEXTURE_COORD_ARRAY); 
		glTexCoordPointer(2,GL_FLOAT,0,textureArray); 
	}
	else if (!TEXTURA && modo!=2) {
		glEnableClientState(GL_COLOR_ARRAY);
		glColorPointer(3,GL_FLOAT,0,vertexArray); 
	}

}

void Render::drawVertexArray(texture textura, float *normals, int begin, int end, RGBA)
{
	if (a<0) glColor3f(r, g, b);
	else {
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
		glColor4f (r, g, b, a);
	}
////////////// setTexture(texture textura) {
        if (textura.bpp==32 && !glIsEnabled(GL_ALPHA_TEST) && TRANSPARENCIA)
        {
                glEnable(GL_ALPHA_TEST);
                glAlphaFunc(GL_GREATER,0.9f);
        } else  
        if (textura.bpp==24 && glIsEnabled(GL_ALPHA_TEST))
        {
                glDisable(GL_ALPHA_TEST);
        }       
        glBindTexture(GL_TEXTURE_2D, textura.idTex);
/////////////// }

	/* texture */
	if (TEXTURA && modo!=2)
		setTexture(textura.idTex);

	if (LIGHT)
	{
		if (normals)
		{
			glNormalPointer(GL_FLOAT,0,normals);
			glEnableClientState( GL_NORMAL_ARRAY );
		}
	}
	
	//Pintamos el mapa
	glDrawArrays(GL_TRIANGLES,begin*3,(end-begin)*3); 

	if (a>=0) glDisable(GL_BLEND);
}

void Render::endVertexArray()
{
	//Desactivamos el vertexArray
	glDisableClientState(GL_VERTEX_ARRAY);

	//Dependiendo del modo que utilizemos desactivamos el GL_TEXTURE_COORD_ARRAY o el
	//GL_COLOR_ARRAY
	if (TEXTURA && modo!=2) glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	else if (!TEXTURA  && modo!=2) glDisableClientState(GL_COLOR_ARRAY);
	glEnableClientState( GL_NORMAL_ARRAY );
}


void Render::drawObject(GLuint textura, float *vertex, float *textureVertex, int numFaces, RGBA)
{
	if (a<0) glColor3f(r, g, b);
	else {
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
		glColor4f (r, g, b, a);
	}

	//Si usamos texturas las activamos y las pintamos.
	//Si no las usamos no las pintamos
	if (TEXTURA && modo!=2)
	{
		//float v[3*sizeof(float)*3];
		//float tv[9*sizeof(float)];

		float v[3*9*numFaces+3];
		float tv[2*9*numFaces+3];

		setTexture(textura);
		glBegin(GL_TRIANGLES);

		for (	register int i=0,j=0,k=0;
			i<numFaces*9;
			i+=3, j+=2, k++)
		{
			if (LIGHT)
			{
			v[3*k+0]=vertex[i+0];
			v[3*k+1]=vertex[i+1];
			v[3*k+2]=vertex[i+2];
			tv[2*k+0]=textureVertex[j];
			tv[2*k+1]=textureVertex[j+1];

			if (k==2)
				{
				k=-1;
				GL3f n = GLFace(
					GL3f(v[0],v[1],v[2]),
					GL3f(v[3],v[4],v[5]),
					GL3f(v[6],v[7],v[8]) ).normal().normalize();
				glNormal3f(n.x,n.y,n.z);
				glTexCoord2f(tv[0],tv[1]);
				glVertex3f(v[0],v[1],v[2]);
				glTexCoord2f(tv[2],tv[3]);
				glVertex3f(v[3],v[4],v[5]);
				glTexCoord2f(tv[4],tv[5]);
				glVertex3f(v[6],v[7],v[8]);
				}
			} else {
				glTexCoord2f(textureVertex[j],textureVertex[j+1]);
				glVertex3f(vertex[i],vertex[i+1],vertex[i+2]);
			}
		}

		glEnd();
	} else {
		glBegin(GL_TRIANGLES);

		if (!TEXTURA && modo!=2) 
			glColor3f(1.0f,0.0f,0.0f);

		for (register int i=0;i<numFaces*9;i+=3)
			glVertex3f(vertex[i],vertex[i+1],vertex[i+2]);

		glEnd();
	}

	if (a>=0) glDisable(GL_BLEND);
}

void Render::drawBoundingCube(GLBounding bounding)
{
	if (!TEXTURA) glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);

	//Si no pintamos solamente las linias que forman los triangulos desactivamos el GL_CULL_FACE
	//porque si no hace un efecto malo.
	//En caso contrario aumentamos la anchura de la linia
	if (DEBUG!=SOLO_VERTICES) 	glDisable(GL_CULL_FACE);
	else glLineWidth(5.0f);

	glColor4f(1.0f,1.0f,0.0f,1.0f);

	//Pintamos el bounding	
	glBegin(GL_QUADS);
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());

		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());

		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());

	glEnd();

	//Lo dejamos todo como antes
	// TODO it doesn't matters
	//if (DEBUG!=SOLO_VERTICES)	glEnable(GL_CULL_FACE);
	//else glLineWidth(2.0f);
	
	glDisable(GL_BLEND);
	if (!TEXTURA) glDisable(GL_TEXTURE_2D);
}
