#include "Room.h"

Room::Room()
{
	init();
}

Room::Room(int id_temp)
{
	init();
	id=id_temp;
}

void Room::init()
{
	vertex = NULL;
	numFaces=0;
	numTextures=0;
	numTextureFaces=0;
	numNormals=0;
	normals = NULL;
	textureVertex=0;
	texturePointer=0;
}

Room::~Room()
{
#if _PVS
	for (int i=0;i<numPortals;i++)
		delete portalObj[i];
	delete[] portalObj;
#endif
	delete[] vertex;
	delete[] textureVertex;
	delete[] texturePointer;
}

#if _PVS
bool Room::createPortals(int tempoPortals)
{
	if (tempoPortals<0) tempoPortals=0;

	numPortals = tempoPortals;

	if (tempoPortals>0)
	{
		portalObj=new Portal*[tempoPortals];
		if (!portalObj) return false;
	}

	return true;
}

void Room::setPortalNeighbour(int where,int vecino)
{
	portalObj[where] = new Portal(vecino);
}

void Room::setPortalVertex(int where,GL3f *vertex_old)
{
	portalObj[where]->setVertex(vertex_old);
}

int Room::getNumPortals()
{
	return numPortals;
}

int Room::getPortalNeighbour(int which)
{
	return portalObj[which]->getId();
}

GL3f *Room::getPortalVertex(int which)
{
	return portalObj[which]->getVertex();
}
#endif


int Room::getId()
{
	return id;
}

void Room::addVertex(float *v,int n)
{
	numFaces=n;

	/* 3 vertex with 3 components (xyz) = *9 */
	delete[] vertex;
	vertex = new float[numFaces*9];

	/* store vector array */
	memcpy(vertex,v,sizeof(float)*numFaces*9);

	/* normals */
	numNormals = numFaces;

	delete[] normals;
	normals = new float[numNormals*3]; // xyz*faces // XXX VALGRIND

	for (register int i=0; i<numNormals ; i++)
	{
		GL3f n = GLFace(
			GL3f(vertex[9*i+0],vertex[9*i+1],vertex[9*i+2]),
			GL3f(vertex[9*i+3],vertex[9*i+4],vertex[9*i+5]),
			GL3f(vertex[9*i+6],vertex[9*i+7],vertex[9*i+8]) ).normal();

		float shared=0;

		for (int j=0;j<numFaces*3;j++)
		{
			if((vertex[3*i+0] == vertex[3*j+0])
			&& (vertex[3*i+1] == vertex[3*j+1])
			&& (vertex[3*i+2] == vertex[3*j+2]) )shared++;
		}
		n=n/shared;
		n=n.normalize();
		normals[(3*i)+0]=n.x;
		normals[(3*i)+1]=n.y;
		normals[(3*i)+2]=n.z;
	}
}

void Room::addTextureVertex(float *texVertex, int nTexFaces)
{
	this->numTextureFaces=nTexFaces;

	/* 3 vertex and 2 components (u,v) */
	delete[] this->textureVertex;
	this->textureVertex=new float[nTexFaces*6];

	/* store array */
	memcpy(textureVertex,texVertex,sizeof(float)*nTexFaces*6);
}

int Room::getNumFaces()
{
	return numFaces;
}

int Room::getNumTextureFaces()
{
	return numTextureFaces;
}

float *Room::getVertex()
{
	return vertex;
}

float *Room::getNormals()
{
	return normals;
}

int Room::getNumNormals()
{
	return numNormals;
}

float *Room::getTextureVertex()
{
	return textureVertex;
}

int Room::getFirstTexture(int which)
{
	if ( which>=0 && which<=numTextures )
		return texturePointer[which*2];
	else 	return -1;
}

int Room::getTextureNumber(int which)
{
	if (which>=0 && which<numTextures) 
		return texturePointer[(which*2)+1];
	else 	return -1;
}

int Room::getNumTextures()
{
	return numTextures;
}

void Room::addTexturePointer(int *tp,int n)
{
	numTextures = n;

	//Multiplicamos por 2 el numero de texturas ya que en el primer campo le defines en que
	//parte del array empiezas a utilizarla y en el segundo a que textura del mapa te refieres.
	//Por ultimo le sumas uno donde colocas la longitud del array
	if (texturePointer) delete[] texturePointer;
	texturePointer = new int[(n*2)+1];

	/* store array */
	memcpy(texturePointer,tp,sizeof(float)*((n*2)+1));
}

