#include "SDL.h"

extern SDL_Surface *WINDOW;

void InitGraphics()
{
	const SDL_VideoInfo* info = NULL;
	FILE *fd;

	chCfgDir();

	/* default values */
	TRANSPARENCIA=1;
	BPP=32;
	FOV=50;
//	FULLSCREEN=0;
	WIDTH=640;
	HEIGHT=480;
	CULL=1;
	STEP=0;

	/* read configuration file */
	fd=fopen("config.vid","r");
	if (fd)
	{
		char buf[101];
		char name[101];
		char *args;
	
		while(!feof(fd))
		{
		if ( fgets(buf,100,fd) == NULL)
			break;
		sscanf(buf,"%s",name);
		if (name[0]=='#') continue; //skip comments

		args=strstr(buf,name)+strlen(name);

		if (!strcmp("TRANS",name)) 
			sscanf(args,"%d",&TRANSPARENCIA);
		if (!strcmp("BPP",name))
			sscanf(args,"%d",&BPP); else
		if (!strcmp("FULLSCREEN",name)) 
		{
			if (FULLSCREEN==0)
			sscanf(args,"%d",&FULLSCREEN); 
		} else
		if (!strcmp("LIGHT",name))
		{
			if (LIGHT==0)
			sscanf(args,"%d",&LIGHT);
		} else
		if (!strcmp("RES",name)) 
			sscanf(args,"%d %d",&WIDTH,&HEIGHT); else
		if (!strcmp("FOV",name)) 
			sscanf(args,"%f",&FOV); else
		if (!strcmp("FOG",name))
			sscanf(args,"%d",&FOG);
		if (!strcmp("CULL",name))
			sscanf(args,"%d",&CULL);
		} // while(!feof(fd))
	
		fclose (fd);
	}

	STEP = WIDTH/10000;

	printf("[i] Initializing SDL\n");

	// Use SDL_INIT_NOPARACHUTE to drop SDL signal handlers
	int err = SDL_Init(SDL_INIT_VIDEO); 
	if ( err < 0 )
	{
        	fprintf(stderr, "Cannot init SDL (%s)\n", SDL_GetError());
	        killProgram (0);
	}
	
	info = SDL_GetVideoInfo( );

	WINDOW = SDL_SetVideoMode (  WIDTH,HEIGHT, BPP,
				     SDL_OPENGL|SDL_HWSURFACE|SDL_DOUBLEBUF|
				     ((FULLSCREEN)?SDL_FULLSCREEN:0));
	if(! WINDOW )
	{
		fprintf(stderr,"ERROR setting video mode.\n");
		killProgram(1);
	}

	SDL_ShowCursor(false);
	SDL_EnableKeyRepeat(1, 0);
	SDL_WM_SetCaption("glH",""); // icon?
	
	/* OpenGL tips */

	glEnable(GL_DEPTH_TEST);
	if (CULL)
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	// glTextures
	glTexEnvi(GL_TEXTURE,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);

	// glHint
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT,  GL_NICEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	SDL_GL_SwapBuffers( );
}

