#include "Sky.h"

Sky::Sky() 
{
	init();
}

Sky::Sky(float x, float y, float z, float width, float height, float length) 
{
	setDimension(x,y,z,width,height,length); 
	init();
}

Sky::~Sky()
{
	if (loaded)
	for (int i=0;i<6;i++)
	{
		TGAloader_untid(tex[i]);
	}
}

void Sky::init()
{
	nextChange=0;
	loaded=false;
	night=GL3f(1,1,1);
	x=0.0f;    width=0.0f;
	y=0.0f;    height=0.0f;
	z=0.0f;    length=0.0f; 
}

// NOT USED (remove?)
void Sky::setDimension(float x, float y, float z, float width, float height, float length) 
{
	this->width=width;
	this->height=height;
	this->length=length; 

	this->x=x-width/2;
	this->y=y-height/2;
	this->z=z-length/2;
}

void Sky::draw(GL3f pos)
{
	if (!loaded) return;

	glPushMatrix();
	glhTranslatef(pos);
	
	if (SDL_GetTicks()>=nextChange)
	{
		nextChange=SDL_GetTicks()+1000;
		tm *timeInfo;
		time_t rawtime;
		time(&rawtime);
		timeInfo=localtime(&rawtime);
		int hour=timeInfo->tm_hour;

		if (hour>=23  || (hour>=0 && hour<=5)) night=GL3f(0.4f,0.3f,0.3f); else
		if (hour==6) night=GL3f(0.7f,0.5f,0.5f); else
		if (hour==7) night=GL3f(0.9f,0.6f,0.6f); else
		if (hour==8) night=GL3f(1.0f,0.8f,0.8f); else
		if (hour>=9 && hour<=11) night=GL3f(1.0f,0.8f,0.8f); else
		if (hour>=12 && hour<=19) night=GL3f(1.0f,1.0f,1.0f); else
		if (hour==20) night=GL3f(0.9f,0.8f,0.8f); else
		if (hour==21) night=GL3f(0.8f,0.5f,0.5f); else
		if (hour==22) night=GL3f(0.5f,0.4f,0.4f);
	}

	glColor3f(night.x,night.y,night.z);
	glEnable(GL_TEXTURE_2D);

	// rotated 90 degreeeee over the y axis
	glBindTexture(GL_TEXTURE_2D,tex[BACK_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y,z);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y + height, z); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,y + height, z);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,y,z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,tex[FRONT_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,y,z + length);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,y + height, z + length);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width , y + height, z + length); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width , y,z + length);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,tex[BOTTOM_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,y,z);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,y,z + length);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,z + length); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y,z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,tex[TOP_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y + height, z);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,y + height,	z + length);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,y + height,	z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,tex[LEFT_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,y + height,z);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,y + height,z + length ); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,y,z + length );
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,y,z );		
	glEnd();

	glBindTexture(GL_TEXTURE_2D,tex[RIGHT_ID]);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y,z );
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y,z + length );
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y + height,z + length ); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y + height,z);
	glEnd();

  glPopMatrix();
}

bool Sky::loadSky(char *ruta)
{
	FILE *fd;
	char tempo[1024];

	loaded=true;

	chdir(getDirs(ruta));

	fd=fopen (ruta,"r");
	if (!fd) return false;

	while ( !feof(fd) )
	{
		tempo[0]=0;
		fscanf(fd,"%s",tempo);
		if (!strcmp(tempo,"POS"))
		{
			fscanf(fd,"%f %f %f",&x,&y,&z);
		} else
		if (!strcmp(tempo,"TAM"))
		{
			fscanf(fd,"%f %f %f",&width,&height,&length);
		}

		if (!strcmp(tempo,"TEXT"))
		{
			for (int i=0;i<6 && !feof (fd);i++)
			{
				fscanf(fd,"%s",tempo);
				if (!strcmp(tempo,"BACK")) {
					fscanf(fd,"%s",tempo);
					tex[BACK_ID]=TGAloader_tid(tempo);
				} else
				if (!strcmp(tempo,"FRONT")) {
					fscanf(fd,"%s",tempo);
					tex[FRONT_ID]=TGAloader_tid(tempo);
				} else
				if (!strcmp(tempo,"BOTTOM")) {
					fscanf(fd,"%s",tempo);
					tex[BOTTOM_ID]=TGAloader_tid(tempo);
				} else
				if (!strcmp(tempo,"TOP")) {
					fscanf(fd,"%s",tempo);
					tex[TOP_ID]=TGAloader_tid(tempo);
				} else
				if (!strcmp(tempo,"LEFT")) {
					fscanf(fd,"%s",tempo);
					tex[LEFT_ID]=TGAloader_tid(tempo);
				} else
				if (!strcmp(tempo,"RIGHT")) {
					fscanf(fd,"%s",tempo);
					tex[RIGHT_ID]=TGAloader_tid(tempo);
				}
			}
		}
	}

	fclose(fd);

	x-=width/2;
	y-=height/2;
	z-=length/2;

	return true;
}
