#ifndef _STAGE_H_
#define _STAGE_H_

#include "OpenGLH.h"
#include "Traces.h"
#include "Item.h"
#include "Camera.h"
#include "Enemy.h"
#include "Mates.h"
#include "Message.h"
#include "Map.h"
#include "Light.h"
#include "Weapon.h"
#include "ParticleSystem.h"

extern void killProgram(int mode);

class Stage
{
	bool thirdperson;
	Message *mensajes;
	bool viewMap;
	float step;
#if _PVS
	int *PVS;
#endif
	Properties **itemObj;
	Object3D **itemModel;
	Weapon **weaponObj;
	int numWeapons;
	//Your player
	Camera *camaraObj;
	Enemy **enemigos;
public:
	Map *mapa;
	Light *lights;
	Traces *trace;

private:
	//Number of items
	int numItems;

	//Number of item models
	int numItemModels;

	//Length of the players array
	int maxPlayers;

	//Number of actual players
	int actualPlayers;


private:
	//Reads the map file. Returns 1 if everything it's ok. Returns 0 if an error happens.
	bool loadMap(char *file);

	//Checks in which room you are. You must pass it your actual position
	int checkCamera(GL3f pos);

#if _PVS
	//Loads the pvs. Waits the PVS file
	int cargaPVS(char *ruta);
#endif
	
public:
	//Inits vars and reads the file info. It waits the direction of a map file.
	Stage(char *,char *);

	//Frees memory
	~Stage();

	//Returns the number of items
	int getNumItems();

	//Returns the item you ask for
	Properties *getItem(int pos);

	//Update everything in the Stage (animations, etc).
	void update();

	void beginFog();
	void endFog();
	// walk effect
	float getWalkEffect();
	void setWalkEffect(float);
	GL2f getCamaraYaw();
	void setCameraVector(GL3f);

	//Returns the id of where you are
	int getCamaraId();

#if _PVS
	//Returns the PVS array of the room you ask for
	int *getPVS(int pos);
#endif

	//Returns your position
	GL3f getCamaraPos();

	//Returns where you are looking at
	GL3f getCamaraObj();

	//Returns your bounding
	GLBounding getCamaraBounding();
	void setThirdPerson(bool);

	//Used to set your position. The first parameter is the position and the second one define
	//if we must check in which room we are
	void setCamaraPos(GL3f posCam, bool collision=true);

	//Changes where you are looking at
	void setCamaraObj(double yawH, double yawV);

	//Changes you hab id
	void setCamaraId(int id);


	//Returns the Message object
	Message *getMessageControl();

	//Returns the map object
	Map *getMap();

	//Adds and enemy. The first parameter is the enemy caracter file, the second one is your nick
	//and the third one is the position in the array to be loaded
	bool addEnemy(char *ruta, char *nick, int pos);

	//Returns the enemy you ask for
	Enemy *getEnemy(int which);

	//Returns the actual number of players
	int getActualPlayers();

	//Returns the max number of players
	int getMaxPlayers();

	//Returns your first view model
	Properties *getCamFirstView();

	//Returns your third view model
	Properties *getCamThirdView();

	//Kills the enemy you ask for
	bool killEnemy(int who);

	//Sets your identification number in the game
	void setCamaraDNI(int num);

	//Returns your identification number in the game
	int getCamaraDNI();

	//Sets your nick
	void setCamaraNick(char *nick);

	//Returns your nick name
	char *getCamaraNick();
	float getRotation();

	bool existsEnemy(int who);
	Weapon *getCamaraWeapon();
	Interface *getCamaraInterface();
	bool getViewMap();
	void swapViewMap();
	Player* getMyBody();
	bool isThirdView();
};

#endif
