#include "Traces.h"

/* FYI: http://astronomy.swin.edu.au/~pbourke/curves/spline/ */

Traces::Traces(int n)
{
	if (n<=0) n=1;
	this->n  = n;
	this->i  = 0;
	this->cp = new float[3*n];
	this->u  = new float[n+DEGREE];
	this->pos = 0;
	this->init= false;
	this->line=(GL3f *)malloc(sizeof(GL3f)*90);

	printf("[i] Map trace of %d control points loaded.\n",n);
}

Traces::~Traces()
{
	delete this->cp;
	delete this->u;
}

void Traces::addControlPoint(float x,float y, float z)
{
	if (i>n) return;

	cp[(i*3)+0] = x;
	cp[(i*3)+1] = y;
	cp[(i*3)+2] = z;
	i++;
}

void Traces::doInit()
{
	/* gen array */
	float p=0;
	for (int j=0;j<90;j++)
	{
		line[j]=getPosition(p);
		line[j].y+=0.1;
		p+=0.10;
	}

	/* gen */
	for (int j=0;j<i+DEGREE;j++)
	{
		if (j < DEGREE) u[j] = 0; else
		if (j <= i) u[j] = j - DEGREE + 1; else
		if (j > i) u[j] = i - DEGREE + 2;
	}
}

float Traces::SplineBlend(int k, int t, float v)
{
	float value;

	if (t == 1) 
	{
		if ((u[k] <= v) && (v < u[k+1]))
			value = 1;
		else
			value = 0;
	}
	else
	{
		if ((u[k+t-1] == u[k]) && (u[k+t] == u[k+1]))
		{
			value = 0;
		} else
		if (u[k+t-1] == u[k]) 
		{
			value = (u[k+t] - v) / (u[k+t] - u[k+1]) * SplineBlend(k+1,t-1,v);
		} else
		if (u[k+t] == u[k+1])
		{
			value = (v - u[k]) / (u[k+t-1] - u[k]) * SplineBlend(k,t-1,v);
		}
		else
		{
			value = (v - u[k]) / (u[k+t-1] - u[k]) * SplineBlend(k,t-1,v) + 
			(u[k+t] - v) / (u[k+t] - u[k+1]) * SplineBlend(k+1,t-1,v);
		}
	}

	return(value);
}

/* gets the vector between the new position and the old one */
GL3f Traces::getObjectiveVector(GL3f xyz)
{
	float next = pos+0.03;
	GL3f two = getPosition(next);
	return two-xyz;
}

void Traces::draw()
{
	if (!init) doInit();

	float p=0;
	glDisable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);

        glPolygonMode(GL_FRONT, GL_FILL);

	glPushMatrix();
	glColor3f(0,1,0);
	glLineWidth(5.0f);
	glBegin(GL_LINE_STRIP);

	for (int j=0;j<90;j++)
	{
		GL3f xyz = line[j];
//printf("%f %f %f\n",xyz.x,xyz.y,xyz.z);
		glVertex3f(xyz.x,xyz.y,xyz.z);
		p+=0.1;
	}
	glEnd();
	glPopMatrix();
}

GL3f Traces::getPosition(float v)
{
	float x,y,z;
	float b; /* power */
	int j;

	x=y=z=0;

	for (j=0;j<i;j++)
	{
		/*
		j = control point to analize
		t = degree ( forced to 3 )
		u = spacing between points
		v = position (ranged by [0..n-t+2]); n=number of points
		*/
		b = SplineBlend(j,DEGREE,v);
		x+=cp[(j*3)+0]*b;
		y+=cp[(j*3)+1]*b;
		z+=cp[(j*3)+2]*b;
	}

	return GL3f(x,y,z);
}


GL3f Traces::getNextPosition()
{
	if (!init) doInit();

	if (pos>=(i-DEGREE+1)) pos=0;

	GL3f xyz = getPosition(pos);
	pos += 0.03;

	return xyz;
}
