#ifndef _INCLUDE_MAIN_H_
#define _INCLUDE_MAIN_H_

#define COLLISION 0

#if COLLISION
	#include <odCollisions.h>
	using namespace od;
#endif

#include "../common/platform.h"
#include "../common/version.h"
#include "../common/items.h"
#include "../common/packet.h"
#include "../common/dirs.h"
#include "../common/crc32.h"
#include "../database/Database.h"

#define CONFIG_FILE "config.svr"


#define CRC_SIZE 4
#define bool char


#if WINDOWS
	#include <windows.h>
	#include <winsock.h>
	#include <malloc.h>
// CAL LINKAR wsock32.lib
	#define close closesocket
	#define chdir SetCurrentDirectory
	#define sleep(x) Sleep(x*1000)
	#define usleep(x) Sleep(x)
#else
	#include <netinet/in.h>
	#include <unistd.h>
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <signal.h>
#endif

#include "universe.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

// Functions definition //
// ==================== //

extern unsigned int crc32(char *buf, long len);
extern int getClient (int s);
void errorHappened(char *str,char *str2);
char *readConfiguration(const char *filename);
char *loadMap(char *filename);
int addItem(int *p,char *file,float x,float y, float z, char *model);
int addWeapon(int *p,char *f, float x, float y, float z);
int addDep(int *p,char *buf);
void closeGlhServer(int signal);
void glh_closeConnection(int me, char *str);
void *databaseWorker(void *arg);


// Weapons structure //
// ================= //

typedef struct {
	int munition,  // Kind of munition
	    damage,    // Damage mod.
	    ammo,      // Default ammo
	    maxammo,   // Max ammo that can wear
	    type,      // 0/1/2 (use 4 animation)
	    speed,     // time between bullet and bullet
	    recoil,    // retroces de l'arma
	    delay;     // Temps entre bala i bala
	float range;   // Weapon action range.
} Weapons;

// Role enums //
// ========== //

enum {	HEAD=0,            //
	BODY,              //
	LEGS,              //
	SHIELD_MAX };

enum {  STRENGTH=0,        // Wpn[0].mod
	SPEED,             // WalkSpeed
	DEFENSE,           // Defense
	LIFE,              // Life points
	AIM,               // punteria
	MOD_MAX };

// RoleData structure //
// ================== //

   typedef struct{
      int munition;  // Munition for this  //a way to make things ugly :(
      int type;      //  type.
      bool haveit;   // Have this weapon?
  } weapons_;

typedef struct {
	int level,         // Level
	    experience;    // Experience
	int base[MOD_MAX]; // Base Data
	int mod[MOD_MAX];  // Mod data
	int shield [SHIELD_MAX];  // Shield (defense+local_shield)
	int weapon,        // actual weapon ID.
	    weaps;         // how many weapons I have.
	weapons_ *weapons;
} RoleData;

// Client_Info structure //
// ===================== //

  typedef struct {
      bool ing;       // waiting for someting
      char what;      // for that
   } wait_;                 // Wait union!

   typedef struct {
      float head;
      float knee;
      float u_body;
      float u_legs;
   } Heights;

typedef struct {
   pthread_mutex_t 
	   lock;      // struct locker
   pthread_t thread;  // thread
   char *rph;         // rph filename
   char car[100];     // car filename
   char *nick;        // nickname
   char *pwd;         // password buf
   bool login;        // user login validated?
   bool onplay;       // is playing
   int sock;          // socket descriptor
   int pdep;          // pointer to deps 
   int lag;           // lag value
   long ID;           // Official user ID
   float x,y,z;       // xyz position
   int ani,rot;       // animation and rotation
   int hab;           // actual room
   unsigned int crc;  // crc buffer
   char key[4];       // key
   wait_ wait;
   RoleData rpg;      // RPG Data structure
   Heights heights;   // Heights of model parts
} Client_info;

typedef struct {
   int n;
   Client_info *p;
} Client;


// Items struct //
// ============ //

typedef struct {
	bool on;            // enabled?
	float x,y,z;        // position
	char pkt[2];        // DO-ACTION
	unsigned int sleep; // Sleep-time to wakeup again
	pthread_t th;       // a thread to sleep x)
} Items;


// Room structure //
// ============== //

typedef struct {
	float x[4],y[4],z[4];
} Portal;


/*
typedef struct {
	float x[3],y[3],z[3];
} Face;*/

typedef struct {
	int portals;
	Portal *port;
	int *pointsto;
	int nfc;
	float *vtx;
	/*
	int faces;
	Face *fcs;*/

} Room;


// Start Points //
// ============ //

typedef struct {
	float x,y,z,r;      // position and rotation
	int hab;            // for PVS
} StartPoint;


// Server configuration //
// ==================== //


typedef struct {
	pthread_mutex_t lock; 	// Mutex lock
	pthread_mutex_t deplock;// Dependence in progress.
	int s;              	// Listen Socket
	int v;              	// Verbose
	unsigned int bsize;     // Buffer size
	bool closing;       	// Server still closing...
	int port;           	// Listen at this port
	int maxnick;        	// Max nick length
	int maxconn;        	// Max number of connection
	int login;          	// Login method
	int chNick;		// Allow change nick
	int kaltime; 		// KeepAlive time to sleep
	int crcEnabled;
	float ratio; 		// Ratio multiplier to the vertex world
	char *login_opt;    	// Login options
	char *map;          	// map filename.
	char *hellomsg;		// Hello Message to the clients
	Database *db;		// Database connection
	pthread_t dbth;		// Database thread.
	// -------------------- // Items
	int items;
	Items *itm;
	// -------------------- // Start Points
	int sp;
	StartPoint *stp;
	// -------------------- // Weapons
	int weaps;
	Weapons *wp;
	// -------------------- // Dependencies
	int pdeps;          	// pointer to the end of deps array.
	int pmcd;               // pointer to minimum common deps xD
	char *dep;          	// file dependencies. [crc][string\0] [...]
	// -------------------- // How many rooms on this map? :)
	int rooms;          	// number of rooms
	Room *room;         	// Room structure data
#if COLISION
	cols::IUniverse* pUnivers;
#endif
} ServerConf;



// Login methods //
// ============= //

enum
{
	LOGIN_ANONYMOUS,
	LOGIN_OFFICIAL,
	LOGIN_CUSTOM
};
extern ServerConf cfg;
extern Client client;

#endif
